/********************************************************************************/
/*                                                                              */
/*    PROGRAM: spiback                                                          */
/*                                                                              */
/*    DESCRIPTION:                                                              */
/*    Calls the main subroutine to generate the detector background response    */
/*    to none, one or several time variable background model components.        */
/*                                                                              */
/*    METHOD:                                                                   */
/*    The background models required are specified by parameters in the         */
/*    input parameter file "spiback.par" from which a FITS data set will be     */
/*    created containing extensions for the count response for each component.  */
/*    In general this will be a table ordered by detector-no, pointing and      */
/*    spectrum energy bin.                                                      */
/*    In general it will be expected that detector background will be modelled  */
/*    by a CONSTANT component and a VARIABLE component to be developed in time  */
/*    from photon event data and spacecraft monitoring instruments.             */
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*    AUTHOR...: P.H.Connell                                                    */
/*    ADDRESS..: Birmingham University - Space Research Group                   */
/*    TELEPHONE: 021 - 4146453                                                  */
/*                                                                              */
/*    HISTORY..:                                                                */
/*    Created 6.11.2002 - Version-4.0                                           */
/*    Changed 27.11.2002 - Version-4.0                                          */
/*                                                                              */
/*            Correction for bug in "back-singles" parameter string.            */
/*                                                                              */
/*            Change of "back-multiples" parameter to three with names          */
/*            "back-multiples-n" to allow shorter input number strings.         */
/*                                                                              */
/*            Removal of "back-psdevents" parameter as its values may           */
/*            taken from the input for parameter "back-singles".                */
/*                                                                              */
/*    Changed 23.12.2002 - Version-4.1                                          */
/*            Add a quadratic fit to count rates for LINEAR model.              */
/*            Forces output for a lINEAR ramp model to be positive.             */
/*                                                                              */
/*    Changed 24.12.2002 - Version-4.2                                          */
/*            Correction to quadratic fit to limit it for 1 or 2 pointings.     */
/*                                                                              */
/*    Changed 27.1.2003  - Version-4.3                                          */
/*            Quadratic polynomial model removed.                               */
/*            Addition of general splied polynomial model.                      */
/*                                                                              */
/*    Changed 14.10.2003 - Version-6.0                                          */
/*            Outputs B-spline components covering any observation              */
/*            period over any time interval for which background varies.        */
/*                                                                              */
/*    EXTERNAL SUBROUTINES:                                                     */
/*    CommonInit   : ISDC initialization of its COMMON area.                    */
/*    CommonExit   : ISDC closing of its COMMON area.                           */
/*    spiback_main : SPIBACK background response generating subroutine.         */
/*                                                                              */
/********************************************************************************/
/* Include any standard system header files */
 
#include <stdlib.h>
#include <stdio.h>
 
/* Include the isdc.h header file */
 
#include <isdc.h>
 
#define ROOTNAME "spiback"	/* Executable name */
#define VERSIONO "6.0"		/* Current version-no */

/*==============================================================================*/
/*   Call up ISDC initializing procedures with <CommonInit>.                    */
/*   Call the main background response generating subroutine <spiback_main>.    */
/*   Close down and exit with <CommonExit>.                                     */
/*==============================================================================*/

 main(int argc, char *argv[])

{

 int  stat;
 char phas[128];
 LogFile LFIL = NULL;

 strcpy(phas," getting execution mode prior to starting main task <spiback_main>");

 stat = CommonInit(ROOTNAME,VERSIONO,argc,argv);

 if ((stat == ISDC_SINGLE_MODE) || (stat == ISDC_SERVER_MODE)) stat = spiback_main(phas,stat); 

/*==============================================================================*/
/*   Close all input/output files referenced by COMMON_PREPARE_PARS             */
/*   say goodbye and exit the program via a black hole called COMMON_EXIT.      */
/*==============================================================================*/
 RILlogMessage(LFIL,Log_0," ");

 if (stat == ISDC_OK)
    { RILlogMessage(LFIL,Log_0,"%s",phas); }
 else
    { RILlogMessage(LFIL,Error_1,"Error %d exiting <spiback_main> while %s",stat,phas); }

 RILlogMessage(LFIL,Log_0," ");
    
 stat = CommonExit(stat);		/* You never come out of here */

 exit;

}
