/********************************************************************/
/*                                                                  */
/*    FUNCTION: spiback_bspline_function                            */
/*                                                                  */
/*    DESCRIPTION:                                                  */
/*    Returns the height of a B-spline component of any order.      */
/*                                                                  */
/*    Given a sequence of B-splining nodes [node=0,1,2...,nmax]     */
/*    separated by a width [wdth] and covering a span [nmax*wdth]   */
/*    the height of the B-spline tailing at node [node] at location */
/*    [leng] from node zero will be returned.                       */
/*                                                                  */
/*    METHOD:                                                       */
/*    The B-spline heights created here are calculated by the       */
/*    recurrence algorithm of Cox and de Boor (1972) as described   */
/*    in "Computational Geometry for Design and Manufacture" from   */
/*    I.D.Faux and M.J.Pratt, Cranfield University Ch 6.2.3 pp 166. */
/*                                                                  */
/*    The recurrence table for a 4th order B-spline "tailing" at    */
/*    the ith node at location [x] is                               */
/*                                                                  */
/*    B(1,i-3,x)                                                    */
/*                   B(2,i-2,x)                                     */
/*    B(1,i-2,x)                    B(3,i-1,x)                      */
/*                   B(2,i-1,x)                   B(4,i,x)          */
/*    B(1,i-1,x)                    B(3,i,x)                        */
/*                   B(2,i,x)                                       */
/*    B(1,i,x)                                                      */
/*                                                                  */
/*    Where the first order splines on the left are                 */
/*                                                                  */
/*    B(1,i,x) = 1/[x(i) - x(i-1)]  if x(i-1) <= x < x(i).          */
/*             = 0                  otherwise.                      */
/*                                                                  */
/*    and the remainder are                                         */
/*                                                                  */
/*    B(m,i,x) = [x-x(i-m)]*B(m-1,i-1,x) + [x(i) - x]*B(m-1,i,x)    */
/*                                                                  */
/*    and the B-spline height returned is B(4,i,x) and x(i) is      */
/*    the coordinate location of the ith node.                      */
/*    The method is general for irregular locations of splining     */
/*    nodes but the subroutine assumes they are regularly spaced.   */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/*    AUTHOR.: P.H.Connell - phc@sal1.sr.back.ac.uk                 */
/*    ADDRESS: University of Birmingham - Space Research Group      */
/*                                                                  */
/*    HISTORY: Created 19.1.2003                                    */
/*                                                                  */
/********************************************************************/
 int spiback_bspline_function
 (
 int     orda,		/* [IN]  B-spline order [1,2,....,16] */
 int     node,		/* [IN]  B-spline function-no in sequence */
 double  Xpos[],	/* [IN]  B-spline knot point sequence */
 double *Leng, 		/* [IN]  Length or location of a point from node=0 */
 double *Hght 		/* [OUT] B-spline height at location [leng] */
 )

{			/* Start of subroutine algorithm */

/*==================================================================*/
/*    W O R K I N G - S T O R A G E   D A T A                       */
/*==================================================================*/
 int    i,m,n;
 double A;
 double X,X1,X2;
 double BFUN[16][16];

/*==================================================================*/
/*   Get first order splines first.                                 */
/*==================================================================*/
 *Hght = 0.0;
 X = *Leng;

 for(n=0; n<orda; n++)		/* Get all splines of order 1 */
    {
    i = node + n;
    X1 = Xpos[i-1];
    X2 = Xpos[i];

    if ( (X >= X1) && (X < X2) )
       { BFUN[0][n] = 1.0/(X2 - X1); }
    else
       { BFUN[0][n] = 0.0; }

    }

/*==================================================================*/
/*  Get ascending order spline values up to order <orda>.           */
/*==================================================================*/
 for(m=2; m<=orda; m++)			/* Scan up higher orders */
    {
    for(n=m; n<=orda; n++)
       {
       i = node + n - 1;
       X1 = Xpos[i-m];
       X2 = Xpos[i];

       A = (X-X1)*BFUN[m-2][n-2] + (X2-X)*BFUN[m-2][n-1];

       if (m == orda)
          { BFUN[m-1][n-1] = A; }
       else if (X2 > X1)
          { BFUN[m-1][n-1] = A/(X2-X1); }
       else
          { BFUN[m-1][n-1] = 0.0; }
       }
    }

 *Hght = BFUN[orda-1][orda-1];		/* Output B-spline value */

 return 0;
}
