/**********************************************************************/
/*                                                                    */
/*     SUBROUTINE: spiback_component_ACS                              */
/*                                                                    */
/*     DESCRIPTION:                                                   */
/*     Returns the counts due to the ACS monitoring system over a     */
/*     single Good Time Interval from count rates stored in the       */
/*     HK archive of type "P__DF__NVTGT__L".                          */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/*     AUTHOR.: P.H.Connell - phc@star.sr.bham.ac.uk                  */
/*     ADDRESS: University of Birmingham - Space Research Group       */
/*                                                                    */
/*     HISTORY: Created 3.12.2002                                     */
/*                                                                    */
/**********************************************************************/

#include <isdc.h>	/* [DEF] ISDC subroutine parameters    */
#include <dal3spi.h>	/* [DEF] DAL3SPI subroutine parameters */

 int spiback_component_ACS
(
 dal_element *ogds,	/* [IN]  Observation parameter data structure */

 double IJDs,		/* [IN]  Observation exposure IJD start time (days) */
 double IJDe,		/* [IN]  Observation exposure IJD end time (days)   */

 OBTime OBTs,		/* [IN]  Observation exposure OBT start time */
 OBTime OBTe,		/* [IN]  Observation exposure OBT end time   */

 double DEADCORR,	/* [IN]  Deadtime correction factor = LIVETIME/ONTIME */

 double *Expo,		/* [OUT] Time integrated exposure (secs) */
 char   *phas		/* [OUT] Subroutine phase/task upon exit */
)

{			/* START OF SUBROUTINE CODE */

/*====================================================================*/
/*     W O R K I N G   S T O R A G E   D A T A   S E C T I O N        */
/*====================================================================*/
    /* const   char   *subr = "spiback_component_ACS"; */
 const   double  Dsecs = 86400.0;		/* Seconds per day */
 static  char   *arch = "P__DF__NVTGT__L";	/* HK archive for ACS data */
 static  long    nofh_maxm = 0;
 static  long   *SHK_VAL;
 static  OBTime *SHK_OBT;

 int     n;
 int     stat;
 int     nofh = 0;
 int     tlly = 0;
 long    nofv = 0;
 long    lint;
 double  Rate;
 LogFile LFIL = NULL;
 
/*====================================================================*/
/*    Get number of ACS sampling values stored in the House Keeping   */
/*    archives for OBT period covering <OBTs,OBTe> and allocate       */
/*    new memory for them if not enough is currently available.       */
/*====================================================================*/
 stat = ISDC_OK;

 strcpy(phas,"getting number of ACS count rates from HK archive ");
 n = strlen(phas);
 strcpy(phas+n,arch);

 stat = DAL3SPIgetNumSCHK(ogds,OBTs,OBTe,&nofh,stat);

 if (stat != ISDC_OK) return stat;

 *Expo = 0.0;
 if (nofh < 1) return stat;		/* No SCHK values so exit */

 if (nofh > nofh_maxm)			/* Not enough memory allocated */
    {
    if (nofh_maxm > 0)			/* Remove current memory */
       {
       free(SHK_VAL);
       free(SHK_OBT);
       }
    SHK_VAL = malloc(2*nofh*sizeof(long));/* Allocate new amount from <nofh> */
    SHK_OBT = malloc(2*nofh*sizeof(OBTime));
    nofh_maxm = nofh;			/* Store current memory allocated */
    }

/*====================================================================*/
/*   Get <nofh> ACS count rates from HK archive over GTI <OBTs,OBTe>. */
/*====================================================================*/
 strcpy(phas,"getting ACS count rates from HK archive ");
 n = strlen(phas);
 strcpy(phas+n,arch);

 stat = DAL3SPIgetSCHK(ogds,arch,DAL_LONG,OBTs,OBTe,SHK_OBT,SHK_VAL,stat);
 if (stat != ISDC_OK) return stat;

/*====================================================================*/
/*   Calculate a mean ACS count rate from nonzero count rates         */
/*   and return a count response over GTI <IJDs,IJDe> in <Expo>.      */
/*====================================================================*/
 tlly = 0;
 nofv = 0;
 Rate = 0.0;
 for(n=0; n<nofh; n++) 			/* Accumulate count rates returned in <SHK_VAL> */
    {
    lint = SHK_VAL[n];

    if (lint > 0)			/* Count rate is positive */
       {
       nofv++;				/* Tally no of nonzero values */
       Rate = Rate + (double)lint;	/* Accumulate nonzero values  */
       }
    else if (lint < 0)			/* Log negative value returned */
       {
       tlly++;				/* Tally no of error messages */
       RILlogMessage(LFIL,Warning_1,
       "Rejecting ACS %s entry-no %d of %d for GTI %f to %f with value %ld.",arch,n+1,nofh,IJDs,IJDe,lint);
       }
    }

 if (nofv > 0) Rate = Rate/nofv;	/* Calculate mean count rate */

 *Expo = 0.000001*(IJDe - IJDs)*Dsecs*Rate/DEADCORR;	/* Counts/1000000 */

 if (tlly > 0) RILlogMessage(LFIL,Log_0," ");	/* Put a space after error message block */

 return stat;
}
