/**********************************************************************/
/*                                                                    */
/*     SUBROUTINE: spiback_component_DFEE                             */
/*                                                                    */
/*     DESCRIPTION:                                                   */
/*     Returns the counts from the DFEE monitoring system stored in   */
/*     the House Keeping archives over a single Good Time Interval    */
/*     and for a single detector.                                     */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/*     AUTHOR.: P.H.Connell - phc@star.sr.bham.ac.uk                  */
/*     ADDRESS: University of Birmingham - Space Research Group       */
/*                                                                    */
/*     HISTORY: Created 3.12.2002                                     */
/*                                                                    */
/**********************************************************************/

#include <isdc.h>		/* [DEF] ISDC subroutine parameters    */
#include <dal3spi.h>		/* [DEF] DAL3SPI subroutine parameters */

 int spiback_component_DFEE
(
 dal_element *ogds,	/* [IN]  Observation parameter data structure */

 int    detn,		/* [IN]  Detector-no [0 -> 18] */

 double IJDs,		/* [IN]  Observation exposure IJD start time (days) */
 double IJDe,		/* [IN]  Observation exposure IJD end time (days)   */

 OBTime OBTs,		/* [IN]  Observation exposure OBT start time */
 OBTime OBTe,		/* [IN]  Observation exposure OBT end time   */

 double LIVETIME,	/* */
 double DTICORR, 	/* [IN]  Deadtime correction factor = LIVETIME/ONTIME */

 double *Expo,		/* [OUT] Time integrated exposure (secs) */
 char   *phas		/* [OUT] Subroutine phase/task upon exit */
)

{			/* START OF SUBROUTINE CODE */

/*====================================================================*/
/*     W O R K I N G   S T O R A G E   D A T A   S E C T I O N        */
/*====================================================================*/
    /* const   char  *subr = "spiback_component_DFEE";
 const   double Dsecs = 86400.0;	        Seconds per day */
 const   long   Countsupperlimit = 400; /* accept only DFEE count rates between these limits. Nominal value should be 1000 [500-1500] */
 const   long   Countslowerlimit = 50;
 static  char  *arch = "P__DF__CAFTS__L";	/* HK archive root for DFEE count rates */
 /* changed from CAFTT to CAFTS */
 static  long   nofh_maxm = 0;
 static  long   *SHK_VAL;
 static  OBTime *SHK_OBT;

 int     n;
 int     stat;
 int     nofh = 0;
 int     tlly = 0;
 long    nofv = 0;
 char 	 type[32] = " ";	/* HK archive type for DFEE count rates */
 long    lint;
 double  Rate;
 LogFile LFIL = NULL;
 
/*====================================================================*/
/*    Get number of DFEE sampling values stored in the House Keeping   */
/*    archives for OBT period covering <OBTs,OBTe> and allocate       */
/*    new memory for them if not enough is currently available.       */
/*====================================================================*/
 stat = ISDC_OK;
 *Expo = 0.0;				/* Init count response returned */
 if (detn > 18) return stat;		/* No DFEE values for pseudo-detectors */

 strcpy(type,arch);
 n = strlen(type);	
 sprintf(type+n,"%d",detn);		/* Append detector-no to DFEE archive type */

 strcpy(phas,"getting number of DFEE count rate values from HK archive ");
 n = strlen(phas);	
 strcpy(phas+n,type);

 stat = DAL3SPIgetNumSCHK(ogds,OBTs,OBTe,&nofh,stat);
 if (stat != ISDC_OK) return stat;

 if (nofh < 1) return stat;		/* No DFEE count rates so exit */

 if (nofh > nofh_maxm)			/* Not enough memory allocated for <nofh> values */
    {
    if (nofh_maxm > 0)			/* Remove current memory */
       {
       free(SHK_VAL);
       free(SHK_OBT);
       }
    SHK_VAL = malloc(2*nofh*sizeof(long));/* Allocate new amount from <nofh> */
    SHK_OBT = malloc(2*nofh*sizeof(OBTime));
    nofh_maxm = nofh;			/* Store current memory allocated */
    }

/*====================================================================*/
/*  Read in <nofh> DFEE count rates for detector <detn>.              */
/*====================================================================*/
 strcpy(phas,"getting DFEE count rates from HK archive ");
 n = strlen(phas);	
 strcpy(phas+n,type);

 stat = DAL3SPIgetSCHK(ogds,type,DAL_LONG,OBTs,OBTe,SHK_OBT,SHK_VAL,stat);
 if (stat != ISDC_OK) return stat;

/*====================================================================*/
/*  Calculate a mean count rate accumulated in <Rate> and return a    */
/*  count response over GTI <IJDs,IJDe> in <Expo>.                    */
/*====================================================================*/
 tlly = 0;
 nofv = 0;
 Rate = 0.0;				/* Init count rate accumulator */

 for(n=0; n<nofh; n++) 			/* Accumulate count rates returned in <SHK_VAL> */
    {
    lint = SHK_VAL[n];
    if (lint > Countslowerlimit && lint < Countsupperlimit)			/* Count rate is inside the meaningful limits */
       {
       nofv++;				/* Tally no of nonzero values */
       Rate = Rate + (double)lint;	/* Accumulate nonzero count rates  */
       }
    else  			        /* Log negative value returned */
       {
       tlly++;				/* Tally no of error messages */
       RILlogMessage(LFIL,Warning_1,
       "Rejecting DFEE %s entry-no %d of %d for GTI %f to %f with value %ld.",type,n+1,nofh,IJDs,IJDe,lint);
       }
    }

 if (nofv > 0) Rate = Rate/nofv;	/* Calculate mean count rate */
 
 *Expo = Rate;	/* Return rate (cts/sec) */

 if (tlly > 0) RILlogMessage(LFIL,Log_0," ");	/* Put a space after error message block */

 return stat;
}
