/**********************************************************************/
/*                                                                    */
/*     SUBROUTINE: spiback_component_IREM                             */
/*                                                                    */
/*     DESCRIPTION:                                                   */
/*     Returns the count response over a single Good Time Interval    */
/*     due to count rates from the IREM monitoring system stored in   */
/*     the HK archives of type "RMC__RATE___Hn" where n = 1,2,3.      */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/*     AUTHOR.: P.H.Connell - phc@star.sr.bham.ac.uk                  */
/*     ADDRESS: University of Birmingham - Space Research Group       */
/*                                                                    */
/*     HISTORY: Created 3.12.2002                                     */
/*                                                                    */
/**********************************************************************/

#include <isdc.h>	/* [DEF] ISDC subroutine parameters    */
#include <dal3hk.h>	/* [DEF] DAL3HK subroutine parameters  */

 int spiback_component_IREM
(
 dal_element *ogds,	/* [IN]  Observation parameter data structure */

 double IJDs,		/* [IN]  Observation exposure IJD start time (days) */
 double IJDe,		/* [IN]  Observation exposure IJD end time (days)   */

 OBTime OBTs,		/* [IN]  Observation exposure OBT start time */
 OBTime OBTe,		/* [IN]  Observation exposure OBT end time   */

 double *Expo,		/* [OUT] Time integrated exposure (secs) */
 char   *phas		/* [OUT] Subroutine phase/task upon exit */
)

{			/* START OF SUBROUTINE CODE */

/*====================================================================*/
/*     W O R K I N G   S T O R A G E   D A T A   S E C T I O N        */
/*====================================================================*/
    /* const   char  *subr = "spiback_component_IREM"; */
 const   double Dsecs = 86400.0;		/* Seconds per day */
 static  char  *arch = "RMC__RATE___H";		/* HK archive root for IREM count rates */
 static  long   nofh_maxm = 0;
 static  long   *SHK_VAL;
 static  OBTime *SHK_OBT;

 int     i,n;
 int     leng;
 int     stat;
 long    nofh = 0;
 long    nofv = 0;
 int     tlly = 0;
 dal_dataType data_type = 0;
 char    type[32] = " ";	/* HK archive type for IREM count rates */
 long    lint;
 double  Rate;
 LogFile LFIL = NULL;
 
/*====================================================================*/
/*  Scan IREM count rates stored in three HK archives to calculate a  */
/*  mean count rate then an integrated count value over <IJDs,IJDe>.  */
/*====================================================================*/
 stat = ISDC_OK;
 Rate = 0.0;				/* Init mean IREM count rate */
 *Expo = 0.0;
 nofv = 0;				/* Init no of nonzero IREM values sampled */

 for(i=1; i<=3; i++)			/* Accumulate rates from three IREM archives */
    {
    strcpy(type,arch);			/* Init with IREM archive root name in HK */
    leng = strlen(arch);
    sprintf(type+leng,"%d",i);		/* Append IREM archive-no to <type> */

    strcpy(phas,"getting number of IREM count rate values in HK archive ");
    n = strlen(phas);
    strcpy(phas+n,type);

    stat = DAL3HKgetValueInfo(ogds,type,INTEGRAL,DAL3HK_RAW,OBTs,OBTe,&nofh,&data_type,stat);

    if (stat != ISDC_OK) return stat;

    if (nofh > 0)			/* There is IREM data of this type */
       {
       if (nofh > nofh_maxm)		/* Not enough memory allocated for <nofh> values */
          {
          if (nofh_maxm > 0)		/* Remove current memory */
             {
             free(SHK_VAL);
             free(SHK_OBT);
             }
          SHK_VAL = malloc(2*nofh*sizeof(long));	/* Allocate new amount for <nofh> values */
          SHK_OBT = malloc(2*nofh*sizeof(OBTime));
          nofh_maxm = nofh;				/* Store current memory allocated */
          }

       strcpy(phas,"getting IREM count rates from HK archive ");
       n = strlen(phas);
       strcpy(phas+n,type);

       stat = DAL3HKgetValues(ogds,type,INTEGRAL,DAL3HK_RAW,OBTs,OBTe,SHK_OBT,SHK_VAL,data_type,stat);

       if (stat != ISDC_OK) return stat;

       tlly = 0;
       for(n=0; n<nofh; n++) 		/* Accumulate <nofh> HK values returned */
          {
          lint = SHK_VAL[n];
          if (lint > 0)
             {
             nofv++;			/* Tally no of nonzero values accumulated */
             Rate = Rate + (double)lint;
             }
          else
             {
             tlly++;			/* Tally no of error messages */
             RILlogMessage(LFIL,Error_1,
             "Rejecting IREM %s entry-no %d of %ld for GTI from %f to %f with value %ld.",
             type,n+1,nofh,IJDs,IJDe,lint);
             }
          }
       if (tlly > 0) RILlogMessage(LFIL,Log_0," ");
       }
    }

 if (nofv > 0) Rate = Rate/nofv;		/* Calculate mean count rate */

 *Expo = 0.001*(IJDe - IJDs)*Dsecs*Rate;	/* Return counts/1000 over <IJDs,IJDe> GTI */

 return stat;
}
