/***********************************************************************/
/*                                                                     */
/*      SUBROUTINE: spiback_component_counts                           */
/*                                                                     */
/*      DESCRIPTION:                                                   */
/*      Returns the integral of a single background time variability   */
/*      function over the exposure time for each pointing, for all     */
/*      detectors and binned count energy bins. [a (D,P,E) array]      */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*      AUTHOR.: P.H.Connell - phc@star.sr.bham.ac.uk                  */
/*      ADDRESS: University of Birmingham - Space Research Group       */
/*                                                                     */
/*      HISTORY: Created 27.5.2000                                     */
/*                                                                     */
/***********************************************************************/

#include <isdc.h>		/* [DEF] ISDC subroutine parameters */
#include <dal3gen.h>		/* [DEF] ISDC DAL3GEN subroutine parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */
 
 int spiback_component_counts
(
 struct  pointing_obs *obsv,	/* [IN]  Observation parameter data structure */

 int     b,			/* [IN]  Background component-no */
 int     s,			/* [IN]  Background component spline-no */

 double  IJDs[],		/* [IN]  Observation exposure IJD start times */
 double  IJDe[],		/* [IN]  Observation exposure IJD end times */
 OBTime  OBTs[],		/* [IN]  Observation exposure OBT start times */
 OBTime  OBTe[],		/* [IN]  Observation exposure OBT end times */
 double  ONTIME[],		/* [IN]  Observation exposure ONTIME intervals (secs) */
 double  LIVETIME[],		/* [IN]  Observation exposure LIVETIME intervals (secs) */
 double  Coeff[],		/* [IN]  Background polynomial model coefficients */
 float   cnts[],		/* [IN]  (D,P,E) array of observation count data */
 float   cvar[],		/* [IN]  (D,P,E) array of observation count variances */

 float   bkgr[]			/* [OUT] (D,P,E) response array to component <b,s> */
)

{				/* START OF SUBROUTINE CODE */

/*====================================================================*/
/*     W O R K I N G   S T O R A G E   D A T A   S E C T I O N        */
/*====================================================================*/
 const   double Dzero = 0.0;
 const   int NO_RESPONSE_IN_ENERGY_BIN = -208005;
 const   int NOT_ENOUGH_POINTINGS = -208007;
 LogFile LFIL = NULL;
 int     stat;
 int     nexpo;
 int     i,j,n,q;
 int     d,p,e;			/* Detector, pointing and energybin counters */
 int     nofv;
 int     dmax;			/* No of detectors to be scanned 1 or ALL */
 double  DTIcorr;		/* Dead time correction factor = LIVETIME/ONTIME */
 double  Expo;			/* Time integrated exposure (days) */
 double  Esum;			/* Time integrated exposure (days) */
 double  Dsum;			/* Total counts in output array <bkgr> */
 float   edif;
 float   resp[DAL_MAX_STRING];
 char    phas[DAL_MAX_STRING];
 char    text[DAL_MAX_STRING];
 char    task[DAL_MAX_STRING] = "<spiback_component_model> returned error";

/*====================================================================*/
/*   Check if GTI sequence is assumed the same for each detector.     */
/*====================================================================*/
 stat = ISDC_OK;
 strcpy(text,obsv->bnam[b-1]);		/* Store component name */

 RILlogMessage(LFIL,Log_0,"");
 if ( obsv->orda[b-1] < 1)		/* Not a splining component */
    { RILlogMessage(LFIL,Log_0,"Creating background response for %s component",text); }
 else
    { RILlogMessage(LFIL,Log_0,"Creating background response for %s spline-no %d",text,s); }

 if (!strncmp(text,"MEAN",4))		/* Return a mean count/pointing model */
    {
    spiback_component_counts_mean(obsv,cnts,cvar,bkgr);
    return 0;
    }

 dmax = obsv->nofd;			/* Get response in all detectors independently */

 if (!strncmp(text,"ACS",3)		/* Get response for one detector if all */
 ||  !strncmp(text,"DFEE",4)		/* have the GTI sequence */
 ||  !strncmp(text,"IREM",4))
     if (!strncmp(&obsv->onegtiseq,"Y",1)) dmax = 1;

/*====================================================================*/
/*   Check for enough pointings to determine polynomial components.   */
/*====================================================================*/
 if ((!strncmp(text,"LIN",3) && (obsv->nofp < 2))
 ||  (!strncmp(text,"QUAD",4) && (obsv->nofp < 3))
 ||  (!strncmp(text,"CUB",3)  && (obsv->nofp < 4))
 ||  (!strncmp(text,"QUAR",4) && (obsv->nofp < 5)))
     {
     RILlogMessage(LFIL,Log_0,"");
     RILlogMessage(LFIL,Log_0,
     "No of pointings %d is too low to determine a %s component",obsv->nofp,text);
     stat = NOT_ENOUGH_POINTINGS;
     return stat;
     }

/*====================================================================*/
/*  Integrate background response for each detector and pointing.     */
/*====================================================================*/
 for(p=0; p<obsv->nofp; p++)		/* Scan pointings */
 {
    n = p*obsv->nofd;			/* Pointer to next exposure block of GTIs */

/*  ------------------------------------------------------------------*/
/*  Scan detectors and check that it has nonzero exposure time.       */
/*  ------------------------------------------------------------------*/
    for(d=0; d<dmax; d++)		/* Scan detectors */
       {
       resp[d] = 0.0;			/* Clear exposure accumulator */

       if (ONTIME[n+d] > 0.0)		/* Correct exposure for LIVETIME */
          {DTIcorr = LIVETIME[n+d]/ONTIME[n+d];}
       else				/* There was no effective exposure */
          {DTIcorr = 0.0;}

/*     ---------------------------------------------------------------*/
/*     Integrate response over <obsv->nofi> Good Time Intervals       */
/*     ---------------------------------------------------------------*/
       if (DTIcorr > 0.0)		/* There was some exposure time */
          {
          Esum = 0.0;			/* Clear exposure accumulator */
	  nexpo = 0;                    /* Clear number of rate values */
          for(i=0; i<obsv->nofi; i++)	/* Scan GTI start/end time pairs which are valid */
             {
             j = n*obsv->nofi + d*obsv->nofi + i;
             if ((IJDs[j] >= 0.0) && (IJDe[j] > 0.0))	/* GTI <i,d,p> is valid */
                {
                stat = spiback_component_model(
                       obsv,b,s,d,
                       IJDs[j],IJDe[j],OBTs[j],OBTe[j],LIVETIME[n+d],DTIcorr,&Expo,phas);

                if (stat != ISDC_OK)	/* Error in DAL3 access subroutines */
                   {
                   RILlogMessage(LFIL,Error_1,"%s %d %s.", task,stat,phas);
                   return stat;
                   }

		if(Expo > 0) 		/* Accumulate mean rates for each GTI start/end time pair */
		  {
		   Esum = Esum + Expo;
		   nexpo++;
		  }
                }
             }
	  if (nexpo > 0) 		/* Store in <resp> as a floating point variable */
             {
	     /* resp[d] = 0.001* ((float)Esum /(float) nexpo)*LIVETIME[n+d]; */
	     resp[d] = 0.001*(float)Esum;
	     }
          }
       }

/*  ------------------------------------------------------------------*/
/*  Store detector count response for pointing-no <p>.                */
/*  ------------------------------------------------------------------*/
    for(d=0; d<obsv->nofd; d++)		/* Scan detectors */
       {
       if (dmax > 1)			/* All detectors scanned */
          { bkgr[n+d] = resp[d]; }
       else				/* Only one detector scanned */
          { bkgr[n+d] = resp[0]; }
       }
 }

/*====================================================================*/
/*  Multiply background response by energy band width.                */
/*====================================================================*/
 nofv = obsv->nofd*obsv->nofp;
 for(e=obsv->nofe; e>=1; e--)		/* Scan count energy bins */
    {

    Dsum = Dzero;
    edif = obsv->eupp[e-1] - obsv->elow[e-1];
    if (edif <= 0.0) edif = 1.0;

    n = 0;
    q = (e-1)*nofv;
    for(p=0; p<obsv->nofp; p++)		/* Scan pointings */
       {
       for(d=0; d<obsv->nofd; d++)	/* Scan detectors */
          {
          bkgr[q] = edif*bkgr[n];	/* Multiply by energy bin width */
          Dsum = Dsum + (double)bkgr[q];
          n++;				/* Next (d,p) exposure time */
          q++;				/* Next (d,p,e) background response value in <bkgr> */
          }
       }

    if ( Dsum == Dzero )		/* Zero count response - send warning */
       {
       stat = NO_RESPONSE_IN_ENERGY_BIN;
       RILlogMessage(LFIL,Warning_1,
       "Warning %d as SPIBACK finds zero response in energy bin %d from %f to %f keV.",
       stat,e,obsv->elow[e-1],obsv->eupp[e-1]);
       RILlogMessage(LFIL,Warning_1,"This could be fatal in some imaging or spectral software");
       stat = ISDC_OK;			/* Continue anyway */
       }
    }

/*====================================================================*/
/*  Reset detectors 19-84 and 85-141 for background rates expected.   */
/*====================================================================*/
 q = 0;					/* Pointer to (d,p) blocks in <bkgr(d,p,e)> */
 for(e=1; e<=obsv->nofe; e++)		/* Scan count energy bins */
    {
    for(p=1; p<=obsv->nofp; p++)	/* Scan pointings */
       {
       Esum = 0.0;
       for(d=1; d<=obsv->nofd; d++)	/* Scan detectors */
          {
          if (d <= 19)			/* Singles */
                {
              Esum = Esum + (double)bkgr[q];
                }
          else if (d <= 85)		/* Multiples */
                {
              bkgr[q] = (float)Esum/19;
                }
          else if (d <= 104)		/* PSD-1 */
                {
              bkgr[q] = bkgr[q-85];
                }
          else if (d <= 123)		/* PSD-2 */
                {
              bkgr[q] = bkgr[q-104];
                }
          else if (d <= 142)		/* PSD-3 */
                {
              bkgr[q] = bkgr[q-123];
                }
          q++;				/* Next (d,p,e) background response entry in <bkgr> */
          }
       }
    }

/*====================================================================*/
/*   Multiply response by relative value for each detector.           */
/*====================================================================*/

 if (!strncmp(text,"DFEE",4)) return stat;	/* But not for DFEE response */

 q = 0;
 for(e=0; e<obsv->nofe; e++)			/* Scan energy bins */
    {
    for(p=0; p<obsv->nofp; p++)			/* Scan pointing bins */
       {
       for(d=0; d<obsv->nofd; d++)		/* Scan detector bins */
          {
          n = obsv->dseq[d];			/* Detector identity-no */
          bkgr[q] = bkgr[q]*obsv->detb[n];	/* Multiply by relative value in <obsv.detb> */
          q++;
          }
       }
    }

 return stat;
}
