/***********************************************************************/
/*                                                                     */
/*      SUBROUTINE: spiback_component_counts_mean                      */
/*                                                                     */
/*      DESCRIPTION:                                                   */
/*      Returns a background model which is proportional to the        */
/*      mean counts per pointing.                                      */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*      AUTHOR.: P.H.Connell - Paul.Connell@uv.es                      */
/*      ADDRESS: University of Valencia - GACE.                        */
/*      HISTORY: Created 14.9.2003                                     */
/*                                                                     */
/***********************************************************************/

#include <isdc.h>		/* [DEF] ISDC subroutine parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */
 
 int spiback_component_counts_mean
(
 struct  pointing_obs *obsv,	/* [IN]  Observation parameter data structure */
 float   pcnt[],		/* [IN]  (D,P,E) observation count data array */
 float   pvar[],		/* [IN]  (D,P,E) observation count variance array */
 float   bkgr[]			/* [OUT] (D,P,E) response array to component */
)

{				/* START OF SUBROUTINE CODE */

/*====================================================================*/
/*     W O R K I N G   S T O R A G E   D A T A   S E C T I O N        */
/*====================================================================*/
 LogFile LFIL = NULL;
 int     stat;
 int     nofd;
 int     d,p,e,q,n;
 double  Dsum;			/* Total counts in output array <bkgr> */
 float   edif;

/*====================================================================*/
/*   Check if GTI sequence is assumed the same for each detector.     */
/*====================================================================*/
 stat = ISDC_OK;
 RILlogMessage(LFIL,Log_0,"Creating a mean count per pointing component");

/*====================================================================*/
/*  Multiply background response by energy band width.                */
/*====================================================================*/
 n = 0;
 for(e=1; e<=obsv->nofe; e++)		/* Scan count energy bins */
    {

    edif = obsv->eupp[e-1] - obsv->elow[e-1];
    if (edif <= 0.0) edif = 1.0;

    for(p=1; p<=obsv->nofp; p++)	/* Scan pointings */
       {

       q = (e-1)*obsv->nofd*obsv->nofp + (p-1)*obsv->nofd;

       nofd = 0;
       Dsum = 0.0;			/* Clear count accumulator */
       for(d=0; d<obsv->nofd; d++)	/* Sum count over all detectors */
          {
          if (pvar[q+d] > 0.0)
             {
             Dsum = Dsum + (double)pcnt[q+d];
             nofd++;
             }
          }
       if (nofd > 0) Dsum = Dsum/nofd;	/* Take mean value */

       for(d=0; d<obsv->nofd; d++)	/* Scan detectors again */
          {
          n++;				/* Next (d,p) exposure time */
          bkgr[n] = edif*(float)Dsum;	/* Multiply by energy bin width */
          }

       }
    }

 return stat;
}
