/**********************************************************************/
/*                                                                    */
/*     SUBROUTINE: spiback_component_model                            */
/*                                                                    */
/*     DESCRIPTION:                                                   */
/*     Returns the counts due to a single time variable background    */
/*     component over some exposure time for one detector over some   */
/*     energy band.                                                   */
/*     The count rate for the component will be either calculated as  */
/*     a function value or extracted from a file prepared by the user */
/*     or an INTEGRAL hopusekeeping file containing ACS, IREM, DFEE   */
/*     values etc and integrated over the exposure time and           */
/*     energy band interval.                                          */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/*     AUTHOR.: P.H.Connell - phc@star.sr.bham.ac.uk                  */
/*     ADDRESS: University of Birmingham - Space Research Group       */
/*                                                                    */
/*     HISTORY: Created 11.11.2002                                    */
/*                                                                    */
/**********************************************************************/

#include <isdc.h>		/* [DEF] ISDC subroutine parameters    */
#include <dal3aux.h>		/* [DEF] DAL3AUX subroutine parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */

 int spiback_component_model
(
 struct pointing_obs *obsv,	/* [IN]  Observation parameter data structure */

 int    b,		/* [IN]  Background component-no */
 int    s,		/* [IN]  Background component spline-no */
 int    detn,		/* [IN]  Detector-no [0 -> 18] */

 double IJDs,		/* [IN]  Observation exposure IJD start time (days) */
 double IJDe,		/* [IN]  Observation exposure IJD end time (days)   */

 OBTime OBTs,		/* [IN]  Observation exposure OBT start time */
 OBTime OBTe,		/* [IN]  Observation exposure OBT end time   */

 double LIVETIME,	/*  */
 double DTIcorr,        /* [IN]  Deadtime correction factor = LIVETIME/ONTIME */

 double *Expo,		/* [OUT] Time integrated exposure (secs) */
 char   *phas		/* [OUT] Subroutine phase/task upon exit */
)

{			/* START OF SUBROUTINE CODE */

/*====================================================================*/
/*     W O R K I N G   S T O R A G E   D A T A   S E C T I O N        */
/*====================================================================*/
 const double Dzero = 0.0;
 const double Dsecs = 86400.0;	/* Seconds per day */
 const double Dpi = 3.141592653589793;
 /* const double Dhalf = 0.5; */
 const char  *subr = "spiback_component_model";
 const int    BACKGROUND_COMPONENT_UNKNOWN = -208004;
 const int    INVALID_BACKGROUND_COMPONENT = -208006;

 int     n;
 /* dal_dataType data_type = 0; */
 double  MJDw;
 double  MJDa,MJDb;
 /* double  MJDt[8];		 MJD sampling times for orbital distance */
 /* double  Dist[8];		 Radial distances from centre of earth */
 double  Freq;			/* Function frequency */
 double  Expt;
 /* double  Rate; */
 double  Inti;
 double  Dval;
 char    type[32] = " ";
 char   *pntr;
 char   *trem;

 int     orda;
 int     nint;
 double  Hght;

 int     stat;
 /* int     nofi = 5;
 int     nofh = 0;
 int     tlly = 0;
 long    nofx = 0;
 long    nofv = 0;
 long    lint; */
 LogFile LFIL = NULL;
 
 char   func[32];	/* [IN]  Background component name */
 double Parm[32];	/* [IN]  Background component parameters */

/*====================================================================*/
/*    Get function type from <func> with value "FUNCTION-TYPE"        */
/*====================================================================*/
 stat = ISDC_OK;

 strcpy(func,obsv->bnam[b-1]);		/* Set component function name */
 for(n=0; n<obsv->nofc[b-1]; n++) Parm[n] = obsv->Parm[b-1][n];

 pntr = strchr(func,'/');
 if (pntr) pntr = strchr(func,'-');
 if (pntr) strcpy(type,pntr+1); else strcpy(type," "); /* Function has no type appended */

/*====================================================================*/
/*  Exposure for a POLYNOMIAL function.                               */
/*====================================================================*/

 if(!strncmp(func,"CON",3)
 || !strncmp(func,"LIN",3) || !strncmp(func,"QUAD",4)
 || !strncmp(func,"CUB",3) || !strncmp(func,"QUAR",4) )
    {
    if (!strncmp(func,"CON",3)) { orda = 1; }		/* 1st order spline component */
    else if (!strncmp(func,"LIN",3)) { orda = 2; }	/* 2nd order spline component */
    else if (!strncmp(func,"QUAD",4)) { orda = 3; }	/* 3rd order spline component */
    else if (!strncmp(func,"CUB",3)) { orda = 4; }	/* 4th order spline component */
    else if (!strncmp(func,"QUAR",4)) { orda = 5; }	/* 5th order spline component */

/*  ------------------------------------------------------------------*/
/*  Integrate exposure for B-spline component over period <IJDs,IJDe> */
/*  ------------------------------------------------------------------*/
    nint = 100;					/* Number of integration intervals covering <IJDs,IJDe> */
    Inti = (IJDe - IJDs)/nint;			/* Length of integration intervals */
    Expt = Dzero; 				/* Init integrated exposure */

    for(n=0; n<nint; n++)			/* Scan <nint> integration intervals on (IJDs,IJDe) */
       {
       Dval = IJDs + (n + 0.5)*Inti;		/* Midpoint of (n+1)th integration interval */
       stat = spiback_bspline_function(orda,obsv->splo[s-1],obsv->Knot,&Dval,&Hght);	/* Get B-spline height */
       Expt = Expt + Hght;			/* Accumulate exposure */
       }

    *Expo  = Expt*Inti*DTIcorr*Dsecs;		/* Modify exposure for deadtime and integration step */

    return stat;
    }

/*====================================================================*/
/*  Exposure for a SINE/SINE+1 or COSINE/COSINE+1 function.           */
/*  Parm[0] = MJD-reference time or phase                             */
/*  Parm[1] = Amplitude                                               */
/*  Parm[2] = Period                                                  */
/*====================================================================*/
 if (!strncmp(func,"SIN",3) || !strncmp(func,"COS",3))
    {
    MJDa = obsv->Mjdref + IJDs - Parm[0];	/* Time relative to MJD in <Parm[0]> */
    MJDb = obsv->Mjdref + IJDe - Parm[0];
    Expt = 0.0;				/* Default exposure to zero */

    if (Parm[2] > 0.0)			/* Period greater than zero */
       {
       Freq = 2*Dpi/Parm[2];		/* Frequency in radians/day */
       if (!strncmp(func,"SIN",3))
          {
          Expt = cos(Freq*MJDa) - cos(Freq*MJDb);
          }
       else
          {
          Expt = sin(Freq*MJDb) - sin(Freq*MJDa);
          }
       Expt = Parm[1]*Dsecs*Expt/Freq;
       }

    if (!strncmp(func+3,"+",1)) Expt = Expt + abs(Parm[1])*Dsecs*(MJDb - MJDa);

    *Expo = DTIcorr*Expt;
    return stat;

    }

/*====================================================================*/
/*  Exposure for an inverse power law based on spacecraft height.     */
/*  Parm[0] = Amplitude                                               */
/*  Parm[1] = Inverse power law index                                 */
/*  Expt = INTEGRAL[ Parm[0]/R(t)**Parm[1] ] from IJDs to IJDe        */
/*====================================================================*/
 if (!strncmp(func,"POW",3))
    {
    stat = spiback_component_radial_POWL(obsv->addr,IJDs,IJDe,Parm,DTIcorr,Expo,phas);
    return stat;
    }

/*====================================================================*/
/*  Get count response for ACS count rates in HK archive.             */
/*====================================================================*/
 if (!strncmp(func,"ACS",3))
    {
    stat = spiback_component_ACS(obsv->addr,IJDs,IJDe,OBTs,OBTe,DTIcorr,Expo,phas);
    return stat;
    }

/*====================================================================*/
/*  Get count response for DFEE count rates in HK archive.            */
/*====================================================================*/
 if (!strncmp(func,"DFEE",4))
    {
    stat = spiback_component_DFEE(obsv->addr,detn,IJDs,IJDe,OBTs,OBTe,LIVETIME,DTIcorr,Expo,phas);
    return stat;
    }

/*====================================================================*/
/*  Get count response for IREM count rates in HK archive.            */
/*====================================================================*/
 if (!strncmp(func,"IREM",4))
    {
    stat = spiback_component_IREM(obsv->addr,IJDs,IJDe,OBTs,OBTe,Expo,phas);
    return stat;
    }

/*====================================================================*/
/*  Exposure time variation function not recongnized.                 */
/*====================================================================*/
 strcpy(phas,"finding algorithm for background model");
 RILlogMessage(LFIL,Error_1,"Background component type %s not recognized by %s.",func,subr);
 stat = BACKGROUND_COMPONENT_UNKNOWN;

 return stat;
}
