/**********************************************************************/
/*                                                                    */
/*     SUBROUTINE: spiback_component_radial_POWL                      */
/*                                                                    */
/*     DESCRIPTION:                                                   */
/*     Returns the background response due to an inverse altitude     */
/*     power law model over a single GTI.                             */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/*     AUTHOR.: P.H.Connell - phc@star.sr.bham.ac.uk                  */
/*     ADDRESS: University of Birmingham - Space Research Group       */
/*                                                                    */
/*     HISTORY: Created 3.12.2002                                     */
/*                                                                    */
/**********************************************************************/

#include <math.h>	/* [DEF] Mathematical functions */
#include <isdc.h>	/* [DEF] ISDC subroutine parameters    */
#include <dal3aux.h>	/* [DEF] DAL3AUX subroutine parameters */

 int spiback_component_radial_POWL
(
 dal_element *ogds,	/* [IN]  Observation parameter data structure */

 double  IJDs,		/* [IN]  Observation exposure IJD start time (days) */
 double  IJDe,		/* [IN]  Observation exposure IJD end time (days)   */

 double  Parm[],	/* [IN]  Background component parameters */
 double  DTIcorr,	/* [IN]  Deadtime correction factor = LIVETIME/ONTIME */

 double *Expo,		/* [OUT] Time integrated exposure (secs) */
 char   *phas		/* [OUT] Subroutine phase/task upon exit */
)

{			/* START OF SUBROUTINE CODE */

/*====================================================================*/
/*     W O R K I N G   S T O R A G E   D A T A   S E C T I O N        */
/*====================================================================*/
 const double Dsecs = 86400.0;	/* Seconds per day */
 const double Dhalf = 0.5;
 const char  *subr = "spiback_component_radial_POWL";

 int     stat;
 int     n;
 double  IJDt[100];		/* IJD sampling times for orbital distance */
 double  Dist[100];		/* Radial distances from centre of earth */
 double  Rate;
 int     nofi = 5;
 LogFile LFIL = NULL;
 
/*====================================================================*/
/*    Exposure for an inverse power law based on spacecraft height    */
/*    Parm[0] = Amplitude                                             */
/*    Parm[1] = Inverse power law index                               */
/*    Rate = INTEGRAL[ Parm[0]/R(t)**Parm[1] ] from IJDs to IJDe      */
/*====================================================================*/
 stat = ISDC_OK;

 if (Parm[1] == 0.0)	/* Variation is flat */
    {
    *Expo = DTIcorr*Parm[0]*(IJDe - IJDs)*Dsecs;
    return stat;
    }

 strcpy(phas,"getting orbital radius");

 nofi = 5;		/* Break interval IJDs -> IJDe into 5 subintervals */

 for(n=0; n<nofi; n++) IJDt[n] = IJDs + n*(IJDe - IJDs)/(nofi-1);

 stat = DAL3AUXgetOrbitDist(ogds,DAL3AUX_ANY,nofi,IJDt,Dist,stat);

 if (stat != ISDC_OK) 
    {
    RILlogMessage(LFIL,Error_1,
    "%s cannot access spacecraft altitude between GTI %f to %f IJD.",subr,IJDs,IJDe);
    return stat;
    }

 Rate = Dhalf*( pow(Dist[0],-Parm[1]) + pow(Dist[nofi-1],-Parm[1]) );

 for(n=1; n<nofi-1; n++) Rate = Rate + pow(Dist[n],-Parm[1]);

 *Expo = DTIcorr*Parm[0]*Rate*(IJDe - IJDs)*Dsecs/(nofi-1);

 return stat;
}
