/*********************************************************************************/
/*                                                                               */
/*     SUBROUTINE: spiback_exposure_splining                                     */
/*                                                                               */
/*     DESCRIPTION:                                                              */
/*     Scans a pointing exposure sequence and returns an integer array of the    */
/*     number of exposures covering each successive flux integation interval     */
/*     and the first exposure pointing-no of each interval.                      */
/*                                                                               */
/*********************************************************************************/
/*                                                                               */
/*     AUTHOR...: P.H.Connell                                                    */
/*     ADDRESS..: University of Valencia - GACE                                  */
/*     TELEPHONE: 0034 96 354 3676                                               */
/*                                                                               */
/*     HISTORY..: Created 21.8.2003                                              */
/*                                                                               */
/*********************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <isdc.h>

int spiback_exposure_splining(

int     nofp,		/* [IN]  No of pointing exposures in <pexp> */
double  MJDs[],		/* [IN]  Exposure MJD start times */
double  MJDe[],		/* [IN]  Exposure MJD end times */

double  Inti,		/* [IN]  Spline sequence integration interval in days */
int     orda,		/* [IN]  No of pointing exposures in <pexp> */

int    *nofs,		/* [OUT] No of spline components */
double  Knot[],		/* [OUT] Spline knot point sequence */
int     splo[])		/* [OUT] Spline knot point location */

{

/*===============================================================================*/
/*     W O R K I N G - S T O R A G E   D A T A                                   */
/*===============================================================================*/
LogFile LFIL = NULL;
int     i,k,n,p,q,s,t;
double  Diff;
int     nofg;
int     pntg_frst[1000];
int     pntg_last[1000];
int     nofi;
double  Splintv;
double  Splimjd;

/*===============================================================================*/
/*   For each energy bin of the count data described in <obsv> search for        */
/*   <irec%nofg> sources via IROS correlation search.                            */
/*===============================================================================*/
RILlogMessage(LFIL,Log_0,"");
RILlogMessage(LFIL,Log_0,"Dividing observation period into splining intervals covering about %f days.",Inti);
RILlogMessage(LFIL,Log_0,"");
RILlogMessage(LFIL,Log_0,"Knot B-spline           Knot points         Integration    Integration");
RILlogMessage(LFIL,Log_0,"-no  -no  Loc                (days)         time (days)    time (secs)");

nofg = 1;				/* Init no of pointing groups/revolutions */
pntg_frst[nofg-1] = 1;
for(p=2; p<=nofp; p++)			/* Scan pointing exposures */
   {
   Diff = MJDs[p-1] - MJDe[p-2];	/* Time jump of exposure <p> beyond previous */
   if (Diff > Inti)			/* Start time of exposure <p> probably new revolution */
      {
      pntg_last[nofg-1] = p - 1;	/* Last pointing in group <nofg> */
      nofg++;				/* Update pointing group counter */
      pntg_frst[nofg-1] = p;		/* First pointing in next group */
      }
   }
pntg_last[nofg-1] = nofp;		/* Don't forget last pointing */

t = 0;
s = 0;				/* Init spline counter */
k = 0;
for(n=0; n<nofg; n++)		/* Scan pointing groups */
   {

   p = pntg_frst[n];
   q = pntg_last[n];
   i = q - p + 1;
   Diff = MJDe[q-1] - MJDs[p-1];	/* Length of pointing group <n> */

   for(i=0; i<orda; i++)		/* Attach extended knot set to start of sequence */
      {
      k++;
      if (n > 0) t++;
      Knot[k-1] = MJDs[p-1];
      RILlogMessage(LFIL,Log_0,"%4d %3d%5d %19f",k,0,t,Knot[k-1]);
      }

   nofi = 1 + (int)(Diff/Inti);		/* No of splining intervals for pointing group <n> */
   Splintv = Diff/nofi;			/* Splining interval for pointing group <n> */
   for(i=1; i<nofi; i++)
      {
      k++;				/* Update knot point counter */
      s++;				/* Update spline counter */
      t++;				/* Update integration interval counter */
      Splimjd = MJDs[p-1] + i*Splintv;	/* Knot point MJD time */
      Knot[k-1] = Splimjd;		/* New knot point */
      splo[s-1] = t;
      RILlogMessage(LFIL,Log_0,"%4d %3d%5d %19f %19f",k,s,t,Splimjd,Splintv);
      }

/* -------------------------------------------------------------------*/
/* Append trailing knotpoint extension to end of group just ended.    */
/* -------------------------------------------------------------------*/
   for(i=0; i<orda; i++)		/* Append extended knot set to end of sequence */
      {
      k++;
      s++;
      t++;
      splo[s-1] = t;
      Knot[k-1] = MJDe[q-1];
      RILlogMessage(LFIL,Log_0,"%4d %3d%5d %19f",k,s,t,Knot[k-1]);
      }
   }

/*===============================================================================*/
/*   Output the list of integration time bins to the logfile.                    */
/*===============================================================================*/
*nofs = s;
RILlogMessage(LFIL,Log_0,"");
RILlogMessage(LFIL,Log_0,"The observation period has an order %d spline over %d intervals.",orda,s);
RILlogMessage(LFIL,Log_0,"");

return 0;
}
