/**********************************************************************/
/*                                                                    */
/*     SUBROUTINE: spiback_exposure_time_intervals                    */
/*                                                                    */
/*     DESCRIPTION:                                                   */
/*     Returns observation exposure time intervals for each pointing  */
/*     direction and detector of SPI.                                 */
/*                                                                    */
/**********************************************************************/

#include <isdc.h>		/* [DEF] ISDC subroutine parameters */
#include <dal3gen.h>		/* [DEF] ISDC DAL3GEN subroutine parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */

 int spiback_exposure_time_intervals
(
 struct pointing_obs *obsv,	/* [IN]  Observation pointing/count header */
 double MJD_sta[],		/* [OUT] Observation exposure MJD start times (days) */
 double MJD_end[],		/* [OUT] Observation exposure MJD end times (days) */
 OBTime OBT_sta[],		/* [OUT] Observation exposure OBT start times */
 OBTime OBT_end[],		/* [OUT] Observation exposure OBT end times */
 double ONTIME[],		/* [OUT] Observation exposure ONTIME interval (secs) */
 double LIVETIME[],		/* [OUT] Observation exposure LIVETIME interval (secs) */
 double MJDs[],			/* [OUT] MJD start times of each exposure */
 double MJDe[],			/* [OUT] MJD end times of each exposure */
 char  *extn,			/* [OUT] Dataset extension name */
 char  *phas,			/* [OUT] Phase at which an error occurred */
 int    instat			/* [IN]  Subroutine entry/exit status */
)

{				/* START OF SUBROUTINE CODE */

/*====================================================================*/
/*     W O R K I N G - S T O R A G E    D A T A                       */
/*====================================================================*/
 int    i,n;
 int    stat;
 int    nofs;			/* Total number of spectra */
 long   nofv;			/* No of values returned */
 int    OBTs_required;		/* Flag to require return of OBTs */
 LogFile LFIL = NULL;
 DAL_ELEMENTP eadd;		/* Extension access address */
 dal_dataType type = DAL_DOUBLE;

/*============================================================================*/
/*    Open "SPI.-OBS.-GTI" extension and get GTIs for each pointing/detector. */
/*    Retrieve detector-no identity sequence from first pointing.             */
/*============================================================================*/
 if (instat != ISDC_OK) return instat;
 stat = ISDC_OK;

 strcpy(phas,"searching OG for pointing exposure goodtime intervals");
 strcpy(extn,"SPI.-OBS.-GTI");
 stat = DALobjectFindElement(obsv->addr,extn,&eadd,stat);
 if (stat != ISDC_OK) return stat;

 OBTs_required = 0;
 for(n=0; n<obsv->nofb; n++)
    {
    if ( !strncmp(obsv->bnam[n],"ACS",3) ) OBTs_required = 1;
    if ( !strncmp(obsv->bnam[n],"DFE",3) ) OBTs_required = 1;
    if ( !strncmp(obsv->bnam[n],"IRE",3) ) OBTs_required = 1;
    }

/*====================================================================*/
/*   Retrieve good time intervals into <MJD_sta, MJD_end, ..>         */
/*====================================================================*/
 nofs = obsv->nofd*obsv->nofp;	/* No of GTI entries */
 n = 0;
 i = 0;
 while(n < nofs)		/* Scan detector x pointing array of GTI entries */
 {
    n++;

    strcpy(phas,"reading TSTART column of GTI data");
    stat = DALtableGetColBins(eadd,"TSTART",0,&type,n,n,&nofv,&MJD_sta[i],stat);
    if (stat != ISDC_OK) return stat;

    strcpy(phas,"reading TSTOP column of GTI data");
    stat = DALtableGetColBins(eadd,"TSTOP",0,&type,n,n,&nofv,&MJD_end[i],stat);
    if (stat != ISDC_OK) return stat;

    if ( OBTs_required )	/* Read in OBTs */
       {
       strcpy(phas,"reading OBT_START column of GTI data");
       stat = DAL3GENtableGetOBTBins(eadd,"OBT_START",0,n,n,&nofv,&OBT_sta[i],stat);
       if (stat != ISDC_OK) return stat;

       strcpy(phas,"reading OBT_END column of GTI data");
       stat = DAL3GENtableGetOBTBins(eadd,"OBT_END",0,n,n,&nofv,&OBT_end[i],stat);
       if (stat != ISDC_OK) return stat;
       }

    strcpy(phas,"reading ONTIME column of GTI data");
    stat = DALtableGetColBins(eadd,"ONTIME",0,&type,n,n,&nofv,&ONTIME[n-1],stat);
    if (stat != ISDC_OK) return stat;

    i = i + obsv->nofi;
 }

/*============================================================================*/
/*   Open "SPI.-OBS.-DTI" extension for LIVETIME for each pointing/detector.  */
/*============================================================================*/
 strcpy(phas,"searching OG for exposure deadtime corrections");
 strcpy(extn,"SPI.-OBS.-DTI");

 stat = DALobjectFindElement(obsv->addr,extn,&eadd,stat);

 if (stat != ISDC_OK) return stat;

/*====================================================================*/
/*   Scan pointing detector groups for LIVETIME in each detector.     */
/*====================================================================*/
 strcpy(phas,"reading in observation LIVETIME column of deadtime data");

 n = 0;
 while(n < nofs)	/* Scan exposure for each detector/pointing */
 {
    n++;
    stat = DALtableGetColBins(eadd,"LIVETIME",0,&type,n,n,&nofv,&LIVETIME[n-1],stat);
    if (stat != ISDC_OK) return stat;

 }

/*====================================================================*/
/*   Scan pointing exposures for start and end times                  */
/*====================================================================*/
 strcpy(phas,"searching the OG for pointing exposure attitude data");
 strcpy(extn,"SPI.-OBS.-PNT");
 stat = DALobjectFindElement(obsv->addr,extn,&eadd,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting pointing start times");
 stat = DALtableGetCol(eadd,"TSTART",0,&type,&nofv,MJDs,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting pointing end times");
 stat = DALtableGetCol(eadd,"TSTOP",0,&type,&nofv,MJDe,stat);
 if (stat != ISDC_OK) return stat;

 return stat;
}
