/*********************************************************************************/
/*                                                                               */
/*     SUBROUTINE:   spiback_main                                                */
/*                                                                               */
/*     DESCRIPTION:                                                              */
/*     Allows the user to choose a time variable background model composed of    */
/*     one or more components and creates a file of relative counts in each      */
/*     detector integrated over each pointing exposure interval and for each     */
/*     energy bin which are defined in the same way as in the binned count data  */
/*     output from SPIHIST.                                                      */
/*                                                                               */
/*     METHOD:                                                                   */
/*     (1) Allocate all data arrays.                                             */
/*                                                                               */
/*     (2) Get name of logfile for messages.                                     */
/*                                                                               */
/*     (3) Read in all input parameters via subroutine "spiback_parameters".     */
/*                                                                               */
/*     (4) Read in exposure times via "spiback_exposure_time_intervals".         */
/*                                                                               */
/*     (5) Integrate background model components over each pointing exposure     */
/*         interval using subroutine "spiback_component_counts".                 */
/*                                                                               */
/*********************************************************************************/
/*                                                                               */
/*     AUTHOR...: P.H.Connell                                                    */
/*     ADDRESS..: University of Valencia - Grupo del Astronomia                  */
/*     TELEPHONE: 021 - 4146453                                                  */
/*                                                                               */
/*     HISTORY..: Created 6.11.2002                                              */
/*                                                                               */
/*              : Changed 11.1.2004                                              */
/*                                                                               */
/*              : Inclusion of output of B-spline components to cover any        */
/*              : time variation interval.                                       */
/*                                                                               */
/*              : Output of each background component after it is created        */
/*              : and not when all are created - saves memory for many splines.  */
/*                                                                               */
/*********************************************************************************/

#include <isdc.h>		/* [DEF] ISDC subroutine parameters */
#include <dal3gen.h>		/* [DEF] DAL3GEN subroutine parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */
 
 int spiback_main(char *phas, int stat)	/* START OF SUBROUTINE */
{

/*===============================================================================*/
/*     O B S E R V A T I O N   D A T A                                           */
/*===============================================================================*/
 struct pointing_obs obsv;	/* Observation parameter data structure */
 double  *MJDs;			/* Observation pointing exposure MJD start times */
 double  *MJDe;			/* Observation pointing exposure MJD end times */
 double  *MJD_sta;		/* Observation pointing/detector exposure MJD start times */
 double  *MJD_end;		/* Observation pointing/detector exposure MJD end times   */
 OBTime  *OBT_sta;		/* Observation pointing/detector exposure OBT start times */
 OBTime  *OBT_end;		/* Observation pointing/detector exposure OBT end times   */
 double  *ONTIME;		/* Observation pointing/detector exposure ONTIME intervals */
 double  *LIVETIME;		/* Observation pointing/detector exposure LIVETIME intervals */
 float   *pcnt;			/* Observation pointing/detector count spectra */
 float   *pvar;			/* Observation pointing/detector count variance values */
 float   *bkgr;			/* Background model detector count response */

/*===============================================================================*/
/*     W O R K I N G - S T O R A G E   D A T A                                   */
/*===============================================================================*/
 int     e,p,d,b,n,q,s;
 int     nofc;
 int     nofo;			/* No of independent output components */
 int     nofs;
 int     retc;
 double  Coeff[16];
 char    extn[16];
 char    line[80];
 char    ofil[DAL_FILE_NAME_STRING] = " ";	/* Name of component data output file */
 LogFile LFIL = NULL;
 const   int NO_RESPONSE_IN_ENERGY_BIN = -208005;

/*==============================================================================*/
/*   Get all input parameters for SPIBACK including a handle <obsv.addr> to     */
/*   access all Observation Datsets for input/output from COMMON_PREPARE_PARS.  */
/*==============================================================================*/
 stat = ISDC_OK;
 RILlogMessage(LFIL,Log_0," ");
 strcpy(phas,"executing program preprocessing with <spiback_parameter_input>.");

 stat = spiback_parameter_input(&obsv,ofil,stat);	/*Read in and check input parameters */

 if (stat != ISDC_OK) return stat;

/*==============================================================================*/
/*   Allocate pointing exposure and background detector,pointing,energy arrays. */
/*==============================================================================*/
 RILlogMessage(LFIL,Log_0,"-----------------------------------------------------");
 RILlogMessage(LFIL,Log_0,"Memory will be allocated for observation data buffers");
 RILlogMessage(LFIL,Log_0,"-----------------------------------------------------");

 nofc = obsv.nofd*obsv.nofp*obsv.nofi;		/* Total no of exposure time start/stop pairs */

 RILlogMessage(LFIL,Log_0,
 "Allocating memory for %d values of MJD start/end times",obsv.nofp);

 MJDs = malloc(obsv.nofp*sizeof(double));	/* Allocate memory for MJD start times */
 MJDe = malloc(obsv.nofp*sizeof(double));	/* Allocate memory for MJD end times */

 RILlogMessage(LFIL,Log_0,
 "Allocating memory for a %d x %d x %d array of (I,D,P) MJD start times",
 obsv.nofi,obsv.nofd,obsv.nofp);

 MJD_sta = malloc(nofc*sizeof(double));		/* Allocate memory for MJD start times */

 RILlogMessage(LFIL,Log_0,
 "Allocating memory for a %d x %d x %d array of (I,D,P) MJD end times",
 obsv.nofi,obsv.nofd,obsv.nofp);

 MJD_end = malloc(nofc*sizeof(double));		/* Allocate memory for MJD end times */

 RILlogMessage(LFIL,Log_0,
 "Allocating memory for a %d x %d x %d array of (I,D,P) OBT start times",
 obsv.nofi,obsv.nofd,obsv.nofp);

 OBT_sta = malloc(nofc*sizeof(OBTime));		/* Allocate memory for OBT start times */

 RILlogMessage(LFIL,Log_0,
 "Allocating memory for a %d x %d x %d array of (I,D,P) OBT end times",
 obsv.nofi,obsv.nofd,obsv.nofp);

 OBT_end = malloc(nofc*sizeof(OBTime));		/* Allocate memory for OBT end times */

 nofc = obsv.nofd*obsv.nofp;			/* Total no of exposure ONTIME/LIVETIME intervals */

 RILlogMessage(LFIL,Log_0,
 "Allocating memory for a %d x %d array of (D,P) ONTIME intervals",
 obsv.nofd,obsv.nofp);

 ONTIME = malloc(nofc*sizeof(double));		/* Allocate memory for exposure ONTIME */

 RILlogMessage(LFIL,Log_0,
 "Allocating memory for a %d x %d array of (D,P) LIVETIME intervals",
 obsv.nofd,obsv.nofp);

 LIVETIME = malloc(nofc*sizeof(double));	/* Allocate memory for exposure LIVETIME */

 nofc = obsv.nofd*obsv.nofp*obsv.nofe;		/* Total no of (D,P,E) count bins */

 pcnt = malloc(nofc*sizeof(float));		/* Allocate memory for (D,P,E) count spectra */

 RILlogMessage(LFIL,Log_0,
 "Allocating memory for a %d x %d x %d array of (D,P,E) count spectra",
 obsv.nofd,obsv.nofp,obsv.nofe);

 pvar = malloc(nofc*sizeof(float));		/* Allocate memory for (D,P,E) count variance spectra */

 RILlogMessage(LFIL,Log_0,
 "Allocating memory for a %d x %d x %d array of (D,P,E) count variance values",
 obsv.nofd,obsv.nofp,obsv.nofe);

 RILlogMessage(LFIL,Log_0,
 "Allocating memory for a %d x %d x %d array of (D,P,E) background component response bins",
 obsv.nofd,obsv.nofp,obsv.nofe);

 bkgr = malloc(nofc*sizeof(float));		/* Allocate memory for one background component count response */

 RILlogMessage(LFIL,Log_0," ");

/*==============================================================================*/
/*   Get observation time intervals for each detector and pointing exposure     */
/*   grouped in <obsv.nofi> good time intervals per exposure.                   */
/*==============================================================================*/
 RILlogMessage(LFIL,Log_0,"<spiback_exposure_time_intervals> will read in exposure good time intervals");
 strcpy(phas,"getting observation exposure time intervals");

 stat = spiback_exposure_time_intervals(
        &obsv,MJD_sta,MJD_end,OBT_sta,OBT_end,
        ONTIME,LIVETIME,MJDs,MJDe,extn,phas,stat);

 if (stat != ISDC_OK) return stat;

/*==============================================================================*/
/*   Group exposures to cover the integration time interval <inti>.             */
/*==============================================================================*/
 RILlogMessage(LFIL,Log_0,"<spiback_exposure_splining> will group exposures into larger integration intervals");
 strcpy(phas,"getting splining group sequence");

 nofo = 0;				/* Init total no of component for output */
 for(b=0; b<obsv.nofb; b++)		/* Look for a splined background model */
    {
    if (obsv.orda[b] > 0)		/* There is a splined background model */
       {
       stat = spiback_exposure_splining(
              obsv.nofp,MJDs,MJDe,obsv.Inti,obsv.orda[b],
             &obsv.nofk,obsv.Knot,obsv.splo);

       nofo = nofo + obsv.nofk;		/* Count background splining components */
       }
    else
       { nofo++; }
    }

/*==============================================================================*/
/*   Read in the count data.                                                    */
/*==============================================================================*/
 RILlogMessage(LFIL,Log_0,"<spiback_observation_counts> will read in detector count data");
 strcpy(phas,"getting count detector/pointing spectra");

 stat = spiback_observation_counts(&obsv,pcnt,pvar,extn,phas,stat);

 if (stat != ISDC_OK) return stat;

 if (!strncmp(&obsv.onegtiseq,"Y",1))
    { RILlogMessage(LFIL,Log_0,"SPIBACK will assume all %d detectors have the same GTI sequence",obsv.nofd); }
 else
    { RILlogMessage(LFIL,Log_0,"SPIBACK will assume all %d detectors have a different GTI sequence",obsv.nofd); }

/*==============================================================================*/
/*  Returns a (D,P,E) array of relative counts for each of <nofo> background    */
/*  model components stored in <bkgr(D,P,E)> before output.                     */
/*  Multiply values by the relative detector background factors in <detb>.      */
/*==============================================================================*/
 RILlogMessage(LFIL,Log_0," ");
 RILlogMessage(LFIL,Log_0,"---------------------------------------------------------------------------------------------");
 RILlogMessage(LFIL,Log_0,"<spiback_component_counts> will create the count response for each background model component");
 RILlogMessage(LFIL,Log_0,"<spiback_output_response>  will output the count response for each background model component");
 RILlogMessage(LFIL,Log_0,"---------------------------------------------------------------------------------------------");


 n = 0;
 b = 0;
 while( (b < obsv.nofb) && (stat == ISDC_OK) )	/* Scan background models */
    {
    b++;					/* Update background model-no */
    nofs = 1;					/* Most models are not splined */
    if ( obsv.orda[b-1] > 0) nofs = obsv.nofk;

    s = 0;
    while( (s < nofs) && (stat == ISDC_OK) )	/* Scan all background model splines */
       {

       strcpy(phas,"creating background component response with <spiback_component_counts>.");
       s++;
       stat = spiback_component_counts(		/* Get exposure values into <bkgr[q]> */
              &obsv,b,s,
              MJD_sta,MJD_end,OBT_sta,OBT_end,ONTIME,LIVETIME,Coeff,pcnt,pvar,
              bkgr);

       if (stat == ISDC_OK)			/* Response returned is not OK */ 
          {
          strcpy(phas,"outputing background component response with <spiback_output_response>.");
          n++;					/* Update component output counter */
          stat = spiback_output_response(&obsv,ofil,n,nofo,b,s,bkgr);	/* Output response in <bkgr> */
          }

       }
    }

 if (stat != ISDC_OK)			/* Response returned is not OK */ 
    {
    retc = 0;
    retc = CommonCloseSWG(obsv.addr,retc);
    return stat;			/* Abort */
    }

/*=========================================================================*/
/*   Store each of the <nofo> background component responses returned in   */
/*   buffer <bkgr(D,P,E,*)> in extensions of the FITS output file <ofil>.  */
/*   Also update any empty keywords in the output OG.                      */
/*=========================================================================*/

 stat = CommonStampObject(obsv.addr,"Background model",stat);
 /* stat = DALattributePutChar(obsv.addr,"CREATOR","spiback-5.0","","",stat); */
 stat = DALattributePutChar(obsv.addr,"INSTRUME","SPI","","",stat);
 stat = DALattributePutChar(obsv.addr,"ISDCLEVL","BKG_I","","",stat);
 /* stat = DALattributePutChar(obsv.addr,"DATAMODE",obsv.datamode,"","",stat);
 stat = DALattributePutChar(obsv.addr,"OGID",obsv.groupid,"","",stat); */
 /* stat = DALattributePutChar(obsv.addr,"OBS_ID",obsv.file,"","",stat); */

/*==============================================================================*/
/*   Close all input/output files referenced by COMMON_PREPARE_PARS             */
/*   say goodbye and exit the program via a black hole called COMMON_EXIT.      */
/*==============================================================================*/
 strcpy(phas,"closing observation group files with <CommonCloseSWG> after successful execution");

 stat = CommonCloseSWG(obsv.addr,stat);

 if (stat != ISDC_OK) return stat;

 strcpy(phas,"Exiting SPIBACK after successful execution");

 return stat;
}
