/***********************************************************************/
/*                                                                     */
/*      SUBROUTINE: spiback_observation_counts                         */
/*                                                                     */
/*      DESCRIPTION:                                                   */
/*      Returns observation counts binned by detector,pointing,energy. */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*      AUTHOR  : P.H.Connell - phc@star.sr.bham.ac.uk                 */
/*      ADDRESS : University of Birmingham - Space Research Group      */
/*                                                                     */
/*      HISTORY : Created 10.11.2002                                   */
/*                                                                     */
/***********************************************************************/

#include <isdc.h>		/* [DEF] ISDC int parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */

 int spiback_observation_counts
(
 struct pointing_obs *obsv,	/* [IN]  Observation pointing/count header */
 float  pcnt[],			/* [OUT] Observation [D,P,E] count spectra */
 float  pvar[],			/* [OUT] Observation [D,P,E] count variance spectra */
 char  *extn,			/* [OUT] Extension name */
 char  *phas,			/* [OUT] Phase at which an error occurred */
 int    stat			/* [I-O] Subroutine entry/exit status */
)

{				/* START OF SUBROUTINE */

/*=====================================================================*/
/*      W O R K I N G - S T O R A G E    D A T A                       */
/*=====================================================================*/
 DAL_ELEMENTP eadd;		/* Extension access address */
 dal_dataType rtyp = DAL_FLOAT;
 dal_dataType ityp = DAL_INT;
 dal_float *rval;
 dal_int *ival;
 static char *subr = "spiback_observation_counts";
 int  d,p,e,q,n;
 int  nofs;
 int  nofc;
 long nofv;
 int NOERRORS = 0;

/*=====================================================================*/
/*    Open at "SPI.-OBS.-DSP" extension and get no of energy bins.     */
/*=====================================================================*/
 if (stat != ISDC_OK) return stat;

 nofs = obsv->nofd*obsv->nofp;		/* Expected no of count spectra */
 nofc = obsv->nofe*nofs;		/* Set total no of count bins   */
 for(q=0; q<nofc; q++)			/* Init output array */
    {
    pcnt[q] = 0.0;
    pvar[q] = 0.0;
    }

 strcpy(phas,"searching the OG for detector/pointing count spectra");
 strcpy(extn,"SPI.-OBS.-DSP");
 stat = DALobjectFindElement(obsv->addr,extn,&eadd,stat);

 if (stat != ISDC_OK)			/* Something is wrong */
    {
    if (stat == DAL_ELEMENT_NOT_FOUND)/* Event data not specified - not fatal */
       {
       RILlogMessage(NULL,Log_0,
       "Event data have not been specified - assumed not to be fatal.");
       stat = ISDC_OK;
       }
    return stat;
    }

/*=====================================================================*/
/*   Get no of count spectra and compare with the no expected.         */
/*=====================================================================*/
 strcpy(phas,"getting no of count spectra");
 stat = DALtableGetNumRows(eadd,&nofv,stat);
 if (stat != ISDC_OK) return stat;

 if (nofv != nofs)			/* They should be the same */
    {
     if (nofv < nofs) stat = TABLE_SHORTER_THAN_EXPECTED;
     else             stat = TABLE_LONGER_THAN_EXPECTED;
     RILlogMessage(NULL,Log_0,"%s finds %d% rows in the %s table when there should be %d",subr,nofv,extn,nofs);
     return stat;
    }

/*=====================================================================*/
/*      Get lower/upper energies of spectrum energy bins.              */
/*=====================================================================*/
 rval = malloc(nofc*sizeof(dal_float));
 ival = malloc(nofc*sizeof(dal_int));

 strcpy(phas,"reading in STAT_ERR spectra");
 stat = DALtableGetCol(eadd,"STAT_ERR",0,&rtyp,&nofv,rval,stat);
 if (stat != ISDC_OK) return stat;

 if (stat != ISDC_OK)			/* Default to Poisson errors below */
    {
    NOERRORS = 1;
    stat = ISDC_OK;
    }
 else					/* Reset errors to variance values */
    {
    NOERRORS = 0;
    q = 0;
    for(e=0; e<obsv->nofe; e++)		/* Scan count energy bins */
       {
       for(p=0; p<obsv->nofp; p++)	/* Scan count pointing bins */
          {
          for(d=0; d<obsv->nofd; d++)	/* Scan count detector bins */
             {
             n = p*obsv->nofd*obsv->nofe + d*obsv->nofe + e;
             pvar[q] = rval[n]*rval[n];
             q++;
             }
          }
       }
    }

/*=====================================================================*/
/*  Get count spectra integer values and output as a real-no vector.   */
/*=====================================================================*/
 strcpy(phas,"Read in COUNT spectra");
 stat = DALtableGetCol(eadd,"COUNTS",0,&ityp,&nofv,ival,stat);
 if (stat != ISDC_OK) return stat;

 q = 0;
 for(e=0; e<obsv->nofe; e++)			/* Scan count energy bins */
    {
    for(p=0; p<obsv->nofp; p++)			/* Scan count pointing bins */
       {
       for(d=0; d<obsv->nofd; d++)		/* Scan count detector bins */
          {
          n = p*obsv->nofd*obsv->nofe + d*obsv->nofe + e;
          pcnt[q] = (float)ival[n];
          if (NOERRORS) pvar[q] = pcnt[q];	/* No errors so default to Poisson */
          q++;
          }
       }
    }

 return stat; 
}
