/***********************************************************************/
/*                                                                     */
/*      SUBROUTINE:  spiback_observation_parameters                    */
/*                                                                     */
/*      DESCRIPTION:                                                   */
/*      Returns observation exposure parameters and count spectrum     */
/*      which is output from the SPIHIST count binning executable.     */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*      AUTHOR  : P.H.Connell - phc@star.sr.bham.ac.uk                 */
/*      ADDRESS : University of Birmingham - Space Research Group      */
/*                                                                     */
/*      HISTORY : Created 10.11.2002                                   */
/*                                                                     */
/***********************************************************************/

#include <isdc.h>		/* [DEF] ISDC int parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */

/*=====================================================================*/
/*    SUBROUTINE INPUT/OUTPUT ARGUMENT DEFINITION                      */
/*=====================================================================*/

 int spiback_observation_parameters
(
 struct pointing_obs *obsv,	/* [OUT] Observation data parameters */
 int    stat			/* [OUT] Subroutine exit status */
)

{				/* START OF SUBROUTINE */

/*=====================================================================*/
/*      W O R K I N G - S T O R A G E                                  */
/*=====================================================================*/
 int  n,q;
 char line[80];			/* Output line of numbers to logfile */
 char task[44] = " in <spiback_observation_parameters> while ";
 char extn[16];			/* Object extension name */
 char phas[64];			/* Access phase at which an error occurred */
 LogFile LFIL = NULL;

/*=====================================================================*/
/*      Retrieve spectrum, pointing and time interval parameters.      */
/*=====================================================================*/
 if (stat != ISDC_OK) return stat;

 RILlogMessage(LFIL,Log_0,"------------------------------------------");
 RILlogMessage(LFIL,Log_0,"Reading in observation exposure parameters");
 RILlogMessage(LFIL,Log_0,"------------------------------------------");
 RILlogMessage(LFIL,Log_0," ");

 stat = spiback_observation_spectrum(obsv,extn,phas,stat);

 stat = spiback_observation_pointing(obsv,extn,phas,stat);

 stat = spiback_observation_goodtime(obsv,extn,phas,stat);

 stat = spiback_observation_deadtime(obsv,extn,phas,stat);

/*=====================================================================*/
/*   Abort on any DAL function error.                                  */
/*=====================================================================*/
 obsv->stat = stat;

 if (stat != ISDC_OK)		/* Abort */
 {
    RILlogMessage(LFIL,Error_1,"Error %d %s %s in dataset type %s.",stat,task,phas,extn);
    return stat;
 }

/*=====================================================================*/
/*  Output observation data detector idenity sequence to logfile.      */
/*=====================================================================*/
 RILlogMessage(LFIL,Log_0,"<spiback_observation_parameters> has found count data for %d detectors, %d pointings and %d energy bins",
 obsv->nofd,obsv->nofp,obsv->nofe);
 RILlogMessage(LFIL,Log_0," ");

 RILlogMessage(LFIL,Log_0,"<spiback_observation_parameters> has found %d start/end time intervals for each GTI entry",obsv->nofi);
 RILlogMessage(LFIL,Log_0," ");

 RILlogMessage(LFIL,Log_0,"Observation count data detector identity sequence is: ");
 RILlogMessage(LFIL,Log_0," ");

 q = 0;
 for(n=1; n<=obsv->nofd; n++)
 {
    sprintf(line+4*q,"%4d",obsv->dseq[n-1]);
    q++;
    if ((q >= 19) || (n == obsv->nofd))
    {
       RILlogMessage(LFIL,Log_0,line);
       q = 0;
    }
 }
 RILlogMessage(LFIL,Log_0," ");

 return stat;
}
