/***********************************************************************/
/*                                                                     */
/*      SUBROUTINE: spiback_observation_pointing                       */
/*                                                                     */
/*      DESCRIPTION:                                                   */
/*      Retrieves observation pointing exposure parameters.            */
/*      them into GALACTIC or ECLIPTIC coordinates if required.        */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*      AUTHOR  : P.H.Connell - phc@star.sr.bham.ac.uk                 */
/*      ADDRESS : University of Birmingham - Space Research Group      */
/*                                                                     */
/*      HISTORY : Created 10.11.2002                                   */
/*                                                                     */
/***********************************************************************/

#include <isdc.h>		/* [DEF] ISDC int parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */

 int spiback_observation_pointing
(
 struct pointing_obs *obsv,	/* [IN]  Observation pointing/count header */
 char  *extn,			/* [OUT] Extension name */
 char  *phas,			/* [OUT] Phase at which an error occurred */
 int    stat			/* [I-O] Subroutine entry/exit status */
)

{				/* START OF SUBROUTINE */

/*=====================================================================*/
/*      W O R K I N G - S T O R A G E    D A T A                       */
/*=====================================================================*/
 const int NO_POINTING_EXPOSURES_FOUND = -210018;
 DAL_ELEMENTP eadd;		/* Extension access address */
 dal_dataType dtyp = DAL_DOUBLE;
 long  nofv;
 char  comm[DAL_MAX_ATT_COMMENT_SIZE];		/* Keyword comment */
 char  unit[DAL_MAX_ATT_UNITS_SIZE];		/* Keyword value units */

/*=============================================================================*/
/*    Open "SPI.-OBS.-PNT" extension, get MJD reference and no of pointings.   */
/*=============================================================================*/
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"searching the OGDI for pointing exposure attitude data");
 strcpy(extn,"SPI.-OBS.-PNT");
 stat = DALobjectFindElement(obsv->addr,extn,&eadd,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting no of pointings");
 stat = DALtableGetNumRows(eadd,&obsv->nofp,stat);
 if (stat != ISDC_OK) return stat;

 if (obsv->nofp < 1)
    {
    strcpy(phas,"finding no pointing sequence");
    stat = NO_POINTING_EXPOSURES_FOUND;
    return stat;
    }

/*=============================================================================*/
/*     Read in observation exposure time keywords.                             */
/*=============================================================================*/
 strcpy(phas,"getting IJD reference time in MJDREF");
 stat = DALattributeGetReal(eadd,"MJDREF",&obsv->Mjdref,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting UTC start-time in DATE-OBS");
 stat = DALattributeGetChar(eadd,"DATE-OBS",obsv->UTCstart,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting UTC end-time in DATE-END");
 stat = DALattributeGetChar(eadd,"DATE-END",obsv->UTCend,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting TSTART keyword");
 stat = DALattributeGetReal(eadd,"TSTART",&obsv->Start,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting TSTOP keyword");
 stat = DALattributeGetReal(eadd,"TSTOP",&obsv->End,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting TELAPSE keyword");
 stat = DALattributeGetReal(eadd,"TELAPSE",&obsv->Elapsed,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

/*=============================================================================*/
/*     Read in observation description, pointings and detector keywords.       */
/*=============================================================================*/
 /* strcpy(phas,"getting OBJECT keyword");
 stat = DALattributeGetChar(eadd,"OBJECT",obsv->desc,unit,comm,stat);
 if (stat != ISDC_OK) stat = ISDC_OK;

 strcpy(phas,"getting OBSERVER keyword");
 stat = DALattributeGetChar(eadd,"OBSERVER",obsv->observer,unit,comm,stat);
 if (stat != ISDC_OK) return stat; */

 /* strcpy(phas,"getting OBS_ID keyword");
 stat = DALattributeGetChar(eadd,"OBS_ID",obsv->obsid,unit,comm,stat);
 if (stat != ISDC_OK) return stat; */

 strcpy(phas,"getting OGID keyword");
 stat = DALattributeGetChar(eadd,"OGID",obsv->groupid,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting ISOC_NUM keyword");
 stat = DALattributeGetInt(eadd,"ISOC_NUM",&obsv->nofw,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting PT_NUM keyword");
 stat = DALattributeGetInt(eadd,"PT_NUM",&obsv->nofp,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

/*=====================================================================*/
/*    Read in observation pointing extent in RA/DEC in degrees.        */
/*=====================================================================*/
 strcpy(phas,"getting RA_MIN keyword");
 stat = DALattributeGetReal(eadd,"RA_MIN",&obsv->RAmin,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting DEC_MIN keyword");
 stat = DALattributeGetReal(eadd,"DEC_MIN",&obsv->DECmin,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting RA_MEAN keyword");
 stat = DALattributeGetReal(eadd,"RA_MEAN",&obsv->RAmid,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting DEC_MEAN keyword");
 stat = DALattributeGetReal(eadd,"DEC_MEAN",&obsv->DECmid,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting RA_MAX keyword");
 stat = DALattributeGetReal(eadd,"RA_MAX",&obsv->RAmax,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting DEC_MAX keyword");
 stat = DALattributeGetReal(eadd,"DEC_MAX",&obsv->DECmax,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

/*=====================================================================*/
/*    Read in observation pointing extent in RA/DEC as text.           */
/*=====================================================================*/
 strcpy(phas,"getting RA_MI_S keyword");
 stat = DALattributeGetChar(eadd,"RA_MI_S",obsv->RAmint,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting DEC_MI_S keyword");
 stat = DALattributeGetChar(eadd,"DEC_MI_S",obsv->DECmint,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting RA_ME_S keyword");
 stat = DALattributeGetChar(eadd,"RA_ME_S",obsv->RAmidt,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting DEC_ME_S keyword");
 stat = DALattributeGetChar(eadd,"DEC_ME_S",obsv->DECmidt,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting RA_MA_S keyword");
 stat = DALattributeGetChar(eadd,"RA_MA_S",obsv->RAmaxt,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting DEC_MA_S keyword");
 stat = DALattributeGetChar(eadd,"DEC_MA_S",obsv->DECmaxt,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

/*=====================================================================*/
/*    Write observation pointing extent in galactic L2/B2 in degrees.  */
/*=====================================================================*/
 strcpy(phas,"getting L2_MIN keyword");
 stat = DALattributeGetReal(eadd,"L2_MIN",&obsv->L2min,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting B2_MIN keyword");
 stat = DALattributeGetReal(eadd,"B2_MIN",&obsv->B2min,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting L2_MEAN keyword");
 stat = DALattributeGetReal(eadd,"L2_MEAN",&obsv->L2mid,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting B2_MEAN keyword");
 stat = DALattributeGetReal(eadd,"B2_MEAN",&obsv->B2mid,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting L2_MAX keyword");
 stat = DALattributeGetReal(eadd,"L2_MAX",&obsv->L2max,unit,comm,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"getting B2_MAX keyword");
 stat = DALattributeGetReal(eadd,"B2_MAX",&obsv->B2max,unit,comm,stat);
 if (stat != ISDC_OK) return stat;


 return stat;
}
