/*=======================================================================================*/
/*                                                                                       */
/*     O B S E R V A T I O N   P O I N T I N G   E X P O S U R E   P A R A M E T E R S   */
/*                                                                                       */
/*     The first data structure here describes the header parameters for the sequence    */
/*     of pointings, count spectra count bins comprising an "observation" while the      */
/*     second describes each "exposure" of the observation by its pointing attitude,     */
/*     start time and interval.                                                          */
/*                                                                                       */
/*=======================================================================================*/

#define MAXNOFKNT 4096	/* Maximum no of time variable background components */
#define MAXNOFBKG 512	/* Maximum no of time variable background components */
#define MAXNOFDET 512	/* Maximum no of detectors */
#define MAXNOFOGF 20	/* Maximum no of OG I-O files */
#define MAXNOFCHA 10000	/* Maximum no of count binning energy channels */
#define TABLE_SHORTER_THAN_EXPECTED -208000
#define TABLE_LONGER_THAN_EXPECTED -208001

/*---------------------------------------------------------------------------------------*/
/*     Structure for observation parameters describing exposure count and pointing bins. */
/*---------------------------------------------------------------------------------------*/
 struct pointing_obs
{
 char   file[256];	/* Observation Group Dataset file name */
 int    stat;		/* Observation Group Dataset file status [0 => OK] */
 dal_element *addr;	/* Observation Group Dataset file address[0 => NOT OPEN] */
 char   desc[128];	/* Observation count file description */
    /* char   observer[128];	Observer identity */
    /* char   obsid[128];   	Observation identifier */
 char   groupid[128]; 	/* Observation group identifier */
 char   datamode[32];	/* Observation data mode */
 char   onegtiseq;	/* Observation GTI sequence same for all detectors [Y/N] */
 char   timesys[32];	/* Time frame system			[] */
 char   timeunit[32];	/* Time units				[days] */
 char   timeref[32];	/* Time reference frame 		[LOCAL etc] */
 char   UTCstart[32];	/* Start time in UTC			[YYYY.MM.DDTHH:MM:SS] */
 char   UTCend[32];	/* End time in UTC			[YYYY.MM.DDTHH:MM:SS] */
 double Mjdref;		/* Observation MJD reference time	[51544.0 days] */
 double Start;		/* Start time after <Mjdref>		[seconds] */
 double End;		/* End time after <Mjdref>		[seconds] */
 double Elapsed;	/* Total elapsed exposure time		[seconds] */
 double Ontime;		/* Total ontime or good time interval	[seconds] */
 double Deadtime;	/* Mean dead time RATIO for all live detectors */

 char   RADECsys[16];	/* Julian coordinate system	[FK5 etc] */
 char   equinox[16];	/* Coordinate system equinox	[2000.0 etc] */
 double RAmin;		/* Minimum observation RA	[degrees] */
 double DECmin;		/* Minimum observation DEC	[degrees] */
 double RAmid;		/* Mean    observation RA	[degrees] */
 double DECmid;		/* Mean    observation DEC	[degrees] */
 double RAmax;		/* Maximum observation RA	[degrees] */
 double DECmax;		/* Maximum observation DEC	[degrees] */
 char   RAmint[32];	/* Minimum observation text RA  */
 char   DECmint[32];	/* Minimum observation text DEC */
 char   RAmidt[32];	/* Mean    observation text RA  */
 char   DECmidt[32];	/* Mean    observation text DEC */
 char   RAmaxt[32];	/* Maximum observation text RA  */
 char   DECmaxt[32];	/* Maximum observation text DEC */
 double L2min;		/* Minimum observation L2 	[degrees] */
 double B2min;		/* Minimum observation B2 	[degrees] */
 double L2mid;		/* Mean    observation L2 	[degrees] */
 double B2mid;		/* Mean    observation B2 	[degrees] */
 double L2max;		/* Maximum observation L2 	[degrees] */
 double B2max;		/* Maximum observation B2 	[degrees] */
 char   coor[16];	/* Observation data coordinate system */
			/* "RA/DEC" - default */
			/* "GALACTIC" */
			/* "ECLIPTIC" - not yet implemented */

 char   detref[32];	/* Reference for pseudo-detector definition */
 long   nofd;		/* Number of detectors (including pseudos) */
 long   nofp;		/* Number of pointing exposure intervals */
 long   nofe;		/* Number of count spectrum energy bins */
 long   nofi;		/* Number of good time intervals per exposure */
 long   nofw;		/* Number of science windows (including slews) */
 long   nofb;		/* Number of background model components */
 long   nofv;		/* Number of background values per model component */

 char   ftyp[MAXNOFOGF][32];	/* Names of OG file types */
 char   fnam[MAXNOFOGF][512];	/* Names of OG input-output files */

 char   bnam[MAXNOFBKG][32];	/* Names of background model components */
 int    orda[MAXNOFBKG];	/* Order of any splining model - usually zero */
 int    nofc[MAXNOFBKG];	/* No of parameters for each component */
 double Parm[MAXNOFBKG][8];	/* Double precison parameters for each component */

 double Inti;			/* Splining interval nominal length */
 int    nofk;			/* No of spline intervals covering observation period */
 int    splo[MAXNOFKNT];	/* Spline knot point location map */
 double Knot[MAXNOFKNT];	/* Spline knot point sequence */

 int    dseq[MAXNOFDET];	/* Detector-no identity sequence in count response buffer */
 float  detb[MAXNOFDET];	/* Detector background multipliers */

 float  emin;		/* Minimum spectrum energy		[keV] */
 float  emax;		/* Maximum spectrum energy		[keV] */
 char   esca;		/* Count spectrum energy scaling	[L=>LOG10] */
 float  elow[MAXNOFCHA];/* Count energy bin lower bound		[keV] */
 float  eupp[MAXNOFCHA];/* Count energy bin upper bound		[keV] */

};

/*---------------------------------------------------------------------------------------*/
/*     Observation pointing exposure data structure for <nofp> sequences.                */
/*---------------------------------------------------------------------------------------*/
 struct pointing_exp
{
 float  axis[2][3];	/* Spacecraft X,Y,Z axis directions	[degrees] */
 float  roll;		/* Roll angle (Clockwise looking up)	[degrees] */
 double Expstart;	/* Exposure start time after <Mjdref> 	[days]    */
 double Expend;		/* Exposure end time after <Mjdref>	[days]    */
 float  exptime;	/* Exposure time interval		[seconds] */
};
