/**********************************************************************/
/*                                                                    */
/*    SUBROUTINE:  spiback_output_component                           */
/*                                                                    */
/*    DESCRIPTION:                                                    */
/*    Outputs the component of each detector for each background      */
/*    model component to a "SPI.-BMOD-DSP" extension.                 */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/*     AUTHOR  : P.H.Connell - phc@sal1.sr.bham.ac.uk                 */
/*     ADDRESS : University of Birmingham - Space Research Group      */
/*                                                                    */
/*     HISTORY : Created 11.11.2002                                   */
/*                                                                    */
/*     EXTERNAL SUBROUTINES: ISDC DAL subroutines                     */
/*                                                                    */
/**********************************************************************/

#include <isdc.h>		/* [DEF] ISDC subroutine parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */
 
 int spiback_output_component
(
 dal_element *eadd,		/* [IN]  Address of background component extension */
 struct pointing_obs *obsv,	/* [IN]  Observation count data parameters */
 char   name[],			/* [IN]  Name of background component */
 float  bkgr[],			/* [IN]  Background model component (D,P,E) response */
 char  *phas			/* [OUT] Last DAL I-O phase */
)

{				/* START OF SUBROUTINE CODE */

/*====================================================================*/
/*     W O R K I N G - S T O R A G E    D A T A                       */
/*====================================================================*/
 LogFile LFIL = NULL;
 int     d,p,e,q,s;
 int     nofs;
 int     stat = 0;
 float   rarr[8192];		/* Array of real numbers */

/*=======================================================*/
/*     Write keyword parameters.                         */
/*=======================================================*/
 if (stat != ISDC_OK) return stat;

 stat = spiback_output_observ_keywords(eadd,obsv,phas);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"writing BKGNAME keyword");
 stat = DALattributePutChar(eadd,"BKGNAME",name,"","",stat);
 if (stat != ISDC_OK) return stat;

/*====================================================================*/
/*   Create columns for storing detector background component.        */
/*====================================================================*/
 strcpy(phas,"creating COUNTS column");
 stat = DALtableAddCol(eadd,"COUNTS",DAL_FLOAT,obsv->nofe,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"creating STAT_ERR column");
 stat = DALtableAddCol(eadd,"STAT_ERR",DAL_FLOAT,obsv->nofe,stat);
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"setting no of spectra in table");
 nofs = obsv->nofp*obsv->nofd;
 stat = DALtableAddRows(eadd,0,nofs,stat);
 if (stat != ISDC_OK) return stat;

/*====================================================================*/
/*   Output count/error response spectra for background component-no  */
/*   <b> to columns "COUNT" and "STAT_ERR" of the output file.        */
/*====================================================================*/
 s = 0;
 for(p=1; p<=obsv->nofp; p++)
    {
    for(d=1; d<=obsv->nofd; d++)
       {
       s++;

/*     ---------------------------------------------------------------*/
/*     Output background response count spectrum for detector (d,p)   */
/*     ---------------------------------------------------------------*/
       strcpy(phas,"writing COUNTS");
       for(e=1; e<=obsv->nofe; e++)
          {
          q = (e-1)*nofs + (p-1)*obsv->nofd + d;
          rarr[e-1] = bkgr[q-1];
          }

       stat = DALtablePutColBins(eadd,"COUNTS",0,DAL_FLOAT,s,s,obsv->nofe,rarr,stat);
       if (stat != ISDC_OK) return stat;

/*     ---------------------------------------------------------------*/
/*     Output background response error spectrum for detector (d,p)   */
/*     ---------------------------------------------------------------*/
       strcpy(phas,"writing STAT_ERR");
       for(e=1; e<=obsv->nofe; e++)
          {
          rarr[e-1] = 0.0;
          }

       stat = DALtablePutColBins(eadd,"STAT_ERR",0,DAL_FLOAT,s,s,obsv->nofe,rarr,stat);
       if (stat != ISDC_OK) return stat;

       }
    }

/*====================================================================*/
/*  Stamp new output dataset and log its successful creation.         */
/*====================================================================*/
 strcpy(phas,"executing CommonStampObject");
 stat = CommonStampObject(eadd,"<spiback_output_component>",stat);

 return stat;
}
