/**********************************************************************/
/*                                                                    */
/*     SUBROUTINE: spiback_output_response                            */
/*                                                                    */
/*     DESCRIPTION:                                                   */
/*     Outputs the SPI detector response to each background model     */
/*     component to a FITS extension of an ISDC grouping file and     */
/*     attaches each to an ISDC index file.                           */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/*     AUTHOR  : P.H.Connell - phc@sal1.sr.bham.ac.uk                 */
/*     ADDRESS : University of Birmingham - Space Research Group      */
/*                                                                    */
/*     HISTORY : Created 11.11.2002                                   */
/*                                                                    */
/*     EXTERNAL SUBROUTINES: ISDC DAL subroutines                     */
/*                                                                    */
/**********************************************************************/

#include <isdc.h>		/* [DEF] ISDC subroutine parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */
#include <dal3gen.h>		/* [DEF] ISDC DAL3GEN subroutine parameters */
 
 int spiback_output_response
(
 struct pointing_obs *obsv,	/* [IN]  Observation parameter data structure */
 char   ofil[],			/* [IN]  Background output component filename */
 int    n,			/* [IN]  Background component-no for output */
 int    nmax,			/* [IN]  Maximum no of components for output */
 int    b,			/* [IN]  Background model-no for output */
 int    s,			/* [IN]  Background model spline-no for output */
 float  bkgr[]			/* [OUT] Background (D,P,E) array of response to component <b,s> */
)

{				/* START OF SUBROUTINE CODE */

/*====================================================================*/
/*     W O R K I N G   S T O R A G E   D A T A   S E C T I O N        */
/*====================================================================*/
 int          stat = 0;
 int          leng = 0;
 LogFile      LFIL = NULL;
 char         name[64];		/* Name of background component */
 char         phas[64];		/* Last DAL I-O phase */
 dal_element *indx; 	 	/* Image catalogue index address */
 dal_element *eadd;		/* Address of background component extension */
 char         subr[40] = "<spiback_output_response>";
 const char  *TEMPLATE_INDX = "SPI.-BMOD-DSP-IDX";
 const char  *TEMPLATE_DATA = "SPI.-BMOD-DSP";

/*====================================================================*/
/*  Locate background model file in observation output group.         */
/*====================================================================*/
 if ( n == 1 )			/* First component for output */
    {
    RILlogMessage(LFIL,Log_0,"");
    RILlogMessage(LFIL,Log_0,"Background response data will be output to file %s",ofil);

    indx = 0;
    strcpy(phas,"locating background model response index in OG");
    stat = DALobjectFindElement(obsv->addr,TEMPLATE_INDX,&indx,stat);

    if (stat == ISDC_OK)
       {
       strcpy(phas,"executing CommonStampObject");
       stat = CommonStampObject(indx,"<spiback_output_response>",stat);
       RILlogMessage(LFIL,Log_0,"");
       }

    if (stat != ISDC_OK)
       {
       RILlogMessage(LFIL,Log_0,"");
       RILlogMessage(LFIL,Error_1,"Error %d returned by %s %s",stat,subr,phas);
       RILlogMessage(LFIL,Log_0,"");
       return stat;
       }
    }

/*====================================================================*/
/*   For each detector background component append a new dataset      */
/*   append it to the index, and fill it with the data in <bkgr>.     */
/*====================================================================*/
 strcpy(phas,"appending a new SPI.-BMOD-DSP entry ");
 stat = DAL3GENindexCreateMember(indx,TEMPLATE_DATA,ofil,&eadd,stat);

 strcpy(name,obsv->bnam[b-1]);
 if (obsv->orda[b-1] > 0)
    {
    leng = strlen(name);
    strcpy(name+leng,"-");
    leng = strlen(name);
    sprintf(name+leng,"%d",s);
    }
 RILlogMessage(LFIL,Log_0,"<spiback_output_component> will output response to background component-no %d to %s.",n,ofil);

 stat = spiback_output_component(eadd,obsv,name,bkgr,phas,stat);

 if (stat == ISDC_OK)		/* Update index for new member */
    {
    strcpy(subr,"<spiback_output_response>");
    strcpy(phas,"updating index with component keywords");
    stat = DAL3GENindexUpdate(eadd,indx,stat);
    }

 if (stat != ISDC_OK)		/* Abort after error output */
    {
    RILlogMessage(LFIL,Error_1,"Error %d returned by %s %s in file %s.",stat,subr,phas,ofil);
    RILlogMessage(LFIL,Error_1,"Error occurred during output of background component %d.",b);
    RILlogMessage(LFIL,Log_0,"");
    return stat;
    }

 RILlogMessage(LFIL,Log_0,
 "Count response to component-no %d called %s has been attached to background model index file.",b,name);
 RILlogMessage(LFIL,Log_0,"");

 if (n < nmax) return stat;		/* Don't close output file until after last component */

 stat = DALobjectClose(indx,DAL_SAVE,stat);

 return stat;
}
