/***********************************************************************/
/*                                                                     */
/*     SUBROUTINE: spiback_parameter_file_check                        */
/*                                                                     */
/*     DESCRIPTION:                                                    */
/*     Checks that all input datasets exist and that output datset     */
/*     do not as they are not to be overwritten.                       */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*     AUTHOR...: P.H.Connell                                          */
/*     ADDRESS..: Birmingham University - Space Research Group         */
/*     TELEPHONE: 021 - 4146453                                        */
/*                                                                     */
/*     HISTORY..:                                                      */
/*     Created 7.11.2002                                               */
/*                                                                     */
/***********************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <isdc.h>		/* [DEF] ISDC subroutine parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */

 int spiback_parameter_file_check
(
 struct pointing_obs *obsv,	/* [I-O] Observation data parameters */
 char  *phas,			/* [OUT] Subroutine execution phase on exit */
 int    stat			/* [I-O] Output status */
)

{				/* START OF SUBROUTINE CODE */

/*=======================================================*/
/*     W O R K I N G - S T O R A G E   D A T A           */
/*=======================================================*/
 char *pntr;
 int   n;
 char  name[DAL_FILE_NAME_STRING] = " ";
 char  fnam[DAL_FILE_NAME_STRING] = " ";
 char  inog[DAL_FILE_NAME_STRING] = " ";
 char  text[DAL_MAX_STRING];
 char  fapp[64];		/* Output file template or DOL number appended */
#define nmax 8
 char  parm[nmax][80];		/* File name parameter list */

/*----------------------------------------------------------------*/
/*   Create list of all input/output file parameter names.        */
/*----------------------------------------------------------------*/
if (stat != ISDC_OK) return stat;

 strcpy(parm[0],"Input observation group dataset [GNRL-OBSG-GRP]: in-og-dol");
 strcpy(parm[1],"Input energy binning dataset [SPI.-EBDS-SET]: ebounds-dol");
 strcpy(parm[2],"Input pointing dataset [SPI.-OBS.-PTG]: pointing-dol");
 strcpy(parm[3],"Input goodtime intervals dataset [SPI.-OBS.-GTI]: gti-dol");
 strcpy(parm[4],"Input deadtime dataset [SPI.-OBS.-DTI]: deadtime-dol");
 strcpy(parm[5],"Input detector event spectra [SPI.-OBS.-DSP]: evts-det-spec-dol");
 strcpy(parm[6],"Output observation group dataset [GNRL-OBSG-GRP] : out-og-dol");
 strcpy(parm[7],"Output background response dataset [SPI.-BMOD-DSP-IDX]: back-mod-idx");

/*----------------------------------------------------------------*/
/*   Check for existence of observation and catalogue datasets.   */
/*----------------------------------------------------------------*/
 RILlogMessage(NULL,Log_0," ");
 RILlogMessage(NULL,Log_0,"------------------------------------------------------------");
 RILlogMessage(NULL,Log_0,"Checking for the existence of all input observation datasets");
 RILlogMessage(NULL,Log_0,"------------------------------------------------------------");

 for(n=0; n<nmax; n++)			/* Scan parameters listed in <parm> */
 {
    strcpy(text,parm[n]);
    pntr = strchr(text,':');		/* Point to parameter name in <parm[n]> */
    strcpy(phas,pntr+2);		/* Copy parameter name to <phas> */
    strcpy(pntr,"\0");			/* Clip off parameter name in <text> */

    stat = PILGetString(phas,name);	/* Read parameter in <phas> */
    if (stat != ISDC_OK) return stat;

    if (strlen(name) > 0)		/* Check file access */
    {
       strncpy(fapp," ",64);
       strncpy(fnam," ",DAL_FILE_NAME_STRING);

       stat = spiback_parameter_file_access(inog,text,phas,name,fnam,fapp);

       if (stat != ISDC_OK) return stat;

       strcpy(obsv->fnam[n],fnam);	/* Save input OG filename */
    }

    pntr = strchr(text,']');		/* Point to parameter name in <parm[n]> */
    strcpy(pntr+1,"\0");		/* Remove everything after "]"*/
    pntr = strchr(text,'[');		/* Point to parameter name in <parm[n]> */
    strcpy(obsv->ftyp[n],pntr);		/* Save OG filetype */

 }

 return stat;
}

/***********************************************************************/
/*                                                                     */
/*     SUBROUTINE: spiback_parameter_file_access                       */
/*                                                                     */
/*     DESCRIPTION:                                                    */
/*     Checks that all input datasets exist and that output datset     */
/*     do not as they are not to be overwritten.                       */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*     AUTHOR...: P.H.Connell                                          */
/*     ADDRESS..: Birmingham University - Space Research Group         */
/*     TELEPHONE: 021 - 4146453                                        */
/*                                                                     */
/*     HISTORY..:                                                      */
/*     Created 7.11.2002                                               */
/*                                                                     */
/***********************************************************************/

 int spiback_parameter_file_access
(
 char *inog,		/* [IN] Input OG filename, if specified */
 char *text,		/* [IN] File description */
 char *parm,		/* [IN] File parameter name */
 char *name,		/* [IN] Name of file */

 char *fnam,		/* [OUT] FITS file name */
 char *fapp 		/* [OUT] DOL number or template name appended to filename <file> */
)

{

/*=======================================================*/
/*    W O R K I N G - S T O R A G E   D A T A            */
/*=======================================================*/
 int   i,k,n;
 int   stat;
 FILE *FPTR;

/*=====================================================================================*/
/*    Check file name has some length.                                                 */
/*=====================================================================================*/
 stat = ISDC_OK;

 if (strlen(name) < 1)			/* No file parameter name */
    {
    stat = DAL_FILE_NOT_ACCESSIBLE;
    RILlogMessage(NULL,Log_0," ");
    RILlogMessage(NULL,Error_1,"Filename parameter %s has no value - retruning error %d.",parm,stat);
    return stat;
    }

/*=====================================================================================*/
/*    Check that input files exist and have a DOL [n] number appended.                 */
/*=====================================================================================*/
 if (!strncmp(text,"I",1))		/* Dataset is for input and should exist */
    {
    i = strcspn(name,"[");
    if (i > 0)				/* A DOL number has been appended */
       {
       strcpy(fapp,name+i);
       strncpy(fnam,name,i);		/* Snip it off to open file */
       }
    else				/* No dol has been attached */
       {
       stat = DAL_FILE_NOT_ACCESSIBLE;
       RILlogMessage(NULL,Log_0," ");
       RILlogMessage(NULL,Log_0,"%s [%s] has no DOL [n] identifier appended to it returning error %d",text,fnam,stat);
       return stat;
       }

    FPTR = fopen(fnam,"r");		/* Try to open file for read only */

    if (FPTR == NULL)			/* File does not exist or is not accessible */
       {
       stat = DAL_FILE_NOT_ACCESSIBLE;
       RILlogMessage(NULL,Log_0," ");
       RILlogMessage(NULL,Error_1,"%s [%s] cannot be accessed - returning error %d",text,fnam,stat);
       }
    else				/* File exists */
       {
       RILlogMessage(NULL,Log_0,"%s [%s] has been found.",text,fnam);
       }

    return stat;

    }

/*=====================================================================================*/
/*    Check that output files do not exist and have creation template names appended   */
/*=====================================================================================*/
 if (!strncmp(text,"O",1))		/* Dataset is for output and should not exist */
    {
    i = strcspn(name,"(");		/* Locate creation template name appended to filename */
    if (i > 0)				/* Template name has been appended to filename */
       {
       strcpy(fapp,name+i);		/* Return template name appended in <fapp> */
       strncpy(fnam,name,i);		/* Snip template off to read file */
       }
    else				/* Template name not appended */
       {
       if (strncmp(parm,"out-og-dol",10))	/* Exit with error if not an output observation group file */
          {
          stat = DAL_FILE_NOT_ACCESSIBLE;
          RILlogMessage(NULL,Log_0," ");
          RILlogMessage(NULL,Error_1,"%s [%s] has no template identifier appended to it - returning error %d.",text,fnam,stat);
          return stat;
          }
       strcpy(fapp,"GNRL-OBSG-GRP");
       }

    FPTR = fopen(fnam,"r");		/* Try to open file for read only */

    if (FPTR != NULL)			/* File seems to exist - abort with an error */
       {
       fclose(FPTR);			/* Close file */
       if (!strncmp(parm,"out-og-dol",10) && !strncmp(inog," ",1))	/* No input OG so this is assumed to be it */
          {
          RILlogMessage(NULL,Log_0,"%s [%s] is assumed to be the OG input file as well.",text,fnam);
          }
       else                            	/* Exit with error */
          {
          stat = DAL_FILE_TOO_BIG;
          RILlogMessage(NULL,Log_0," ");
          RILlogMessage(NULL,Error_1,"%s [%s] already exists and should be deleted - returning error %d.",text,fnam,stat);
          return stat;
          }
       }

    else if (!strncmp(parm,"out-og-dol",10))	/* Output OG does not exist and COMMON_PREPARE_PAR will create it */
       {
       RILlogMessage(NULL,Log_0,"%s [%s] will be created by ISDC subroutine COMMON_PREPARE_PAR.",text,fnam);
       }

    else					/* File does not exist and will be created with template appended */
       {
       RILlogMessage(NULL,Log_0,"%s [%s] will be created with template type %s.",text,fnam,fapp);
       }
    }

 return stat;
}
