/***********************************************************************/
/*                                                                     */
/*     SUBROUTINE:  spiback_parameter_input                            */
/*                                                                     */
/*     DESCRIPTION:                                                    */
/*     Returns all input parameters for "spiback" generation of the    */
/*     detector response to a constant and several time variable       */
/*     background model components.                                    */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*     AUTHOR...: P.H.Connell                                          */
/*     ADDRESS..: Birmingham University - Space Research Group         */
/*     TELEPHONE: 021 - 4146453                                        */
/*                                                                     */
/*     HISTORY..:                                                      */
/*                                                                     */
/*     Created 7.11.2002                                               */
/*                                                                     */
/***********************************************************************/

#include <isdc.h>		/* [DEF] ISDC subroutine parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */

/*=====================================================================*/
/*   SUBROUTINE INPUT/OUTPUT ARGUMENT DEFINITION                       */
/*=====================================================================*/
 int spiback_parameter_input
(
 struct pointing_obs *obsv,	/* [OUT] Observation data parameters */
 char   ofil[],			/* [OUT] Filename of background component output dataset */
 int    stat			/* [OUT] Subroutine status on exit */
)

{				/* START OF SUBROUTINE */

/*=====================================================================*/
/*    W O R K I N G - S T O R A G E   D A T A                          */
/*=====================================================================*/
 const int  DAL_TEMPLATE_NOT_FOUND = -1366;	/* This shouldn"t be here   */
 char  phas[80];				/* Subroutine phase on exit */
 const char *idsi = "in-og-dol";		/* Input Observation Group for imaging  */
 const char *odsi = "out-og-dol";		/* Output Observation Group for imaging */
 const char *ipar = "ebounds-dol,pointing-dol,gti-dol,deadtime-dol,evts-det-spec-dol"; /* Input datasets */
 const char *opar = "back-mod-idx";		/* Output datasets */
 int   maku = 1;
 int   clob = 0;
 LogFile LFIL = NULL;

/*=====================================================================*/
/*    Get OG dataset names and check for their existence.              */
/*    Get all input datasets from the Observation Group Datasets for   */
/*    input created by some prior process like "spihist" etc along     */
/*    output datasets via the COMMON_PREPARE_PARS subroutine.          */
/*=====================================================================*/

 stat = spiback_parameter_file_check(obsv,phas,stat);	/* Check all I-O datasets */ 

 RILlogMessage(LFIL,Log_0,"---------------------------------------------------------");
 RILlogMessage(LFIL,Log_0,"Creating output observation group with COMMON_PREPARE_PAR");
 RILlogMessage(LFIL,Log_0,"---------------------------------------------------------");

 stat = CommonPreparePARsStrings(idsi,ipar,odsi,opar,maku,&obsv->addr,&clob,stat);

 RILlogMessage(LFIL,Log_0," ");

/*---------------------------*/
/*    Anyone for errors?     */
/*---------------------------*/
 if (stat == DAL_FILE_NOT_ACCESSIBLE)
    {
    RILlogMessage(LFIL,Error_1,"<spiback_parameter_input> has returned error %d trying to access an input datset.",stat);
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"This error was returned from COMMON_PREPARE_PAR creating an output OG.");
    RILlogMessage(LFIL,Log_0,"One of the input datasets associated with the parameters listed below cannot be accessed.");
    RILlogMessage(LFIL,Log_0,"Check their names to see they are correct and exist.");
    RILlogMessage(LFIL,Log_0,"Note that DOL input datasets names should end with .fits[n] where n=1,2,3....");
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"Input dataset parameter names: %s",ipar);
    return stat;
    }

 if (stat == DAL_CREATE_ERROR)
    {
    RILlogMessage(LFIL,Error_1,"<spiback_parameter_input> has returned error %d trying to create an output datset.",stat);
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"This error was returned from COMMON_PREPARE_PAR creating an output OG.");
    RILlogMessage(LFIL,Log_0,"One of the output datasets associated with the parameters listed below cannot be created.");
    RILlogMessage(LFIL,Log_0,"Check if any exist and delete them because they can not be overwritten.");
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"Output dataset parameter names: %s",opar);
    return stat;
    }

 if (stat == PIL_NOT_FOUND) 
    {
    RILlogMessage(LFIL,Error_1,"<spiback_parameter_input> has returned error %d accessing a non-existent input parameter",stat);
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"This error was returned from COMMON_PREPARE_PAR creating an output OG.");
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"Input dataset parameter names: %s",ipar);
    RILlogMessage(LFIL,Log_0,"Output dataset parameter names: %s",opar);
    return stat;
    }

 if (stat == DAL_TEMPLATE_NOT_FOUND) 
    {
    RILlogMessage(LFIL,Error_1,"<spiback_parameter_input> has returned error %d accessing a template for an output file",stat);
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"This error was returned from COMMON_PREPARE_PAR creating an output OG.");
    RILlogMessage(LFIL,Log_0,"Check the template names for the output datasets associated with the following parameter names: ");
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"Output dataset parameter names: %s",opar);
    return stat;
    }

 if (stat != ISDC_OK)
    {
    RILlogMessage(LFIL,Error_1,"<spiback_parameter_input> has returned error %d from COMMON_PREPARE_PAR while creating an output OG.",stat);
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"Input dataset parameter names: %s",ipar);
    RILlogMessage(LFIL,Log_0,"Output dataset parameter names: %s",opar);
    return stat;
    }

/*==============================================================================*/
/*   Read in observation exposure parameters and store in structure <obsv>.     */
/*   Read in and check all input parameters in "spinack.par".                   */
/*==============================================================================*/
 strcpy(phas,"reading in observation parameters with <spiback_observation_parameters>.");

 stat = spiback_observation_parameters(obsv,stat);

 stat = spiback_parameter_values(obsv,ofil,phas,stat); 	/* Get all other parameters */

 if (stat == PIL_NOT_FOUND)		/* Output no-parameter message to logfile */
    {
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"Parameter [%s] was not specified.",phas);
    RILlogMessage(LFIL,Log_0," ");
    return stat;
    }

 if (stat != ISDC_OK) 			/* Output a general error message to logfile */
    {
    RILlogMessage(LFIL,Log_0," ");
    RILlogMessage(LFIL,Log_0,"Error %d returned while reading input parameter [%s].",stat,phas);
    return stat;
    }

 RILlogMessage(LFIL,Log_0,"Input parameters have been read in and correctly processed");
 RILlogMessage(LFIL,Log_0," ");

 return stat;

}			/* END OF SUBROUTINE */
