/***********************************************************************/
/*                                                                     */
/*     SUBROUTINE:  spiback_parameter_values                           */
/*                                                                     */
/*     DESCRIPTION:                                                    */
/*     Returns input parameters for the selection of the components    */
/*     of some background model for the calculation and output of      */
/*     absolute or relative background rates for each detector and     */
/*     its pointing and energy channel.                                */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*     AUTHOR...: P.H.Connell                                          */
/*     ADDRESS..: Birmingham University - Space Research Group         */
/*     TELEPHONE: 021 - 4146453                                        */
/*                                                                     */
/*     HISTORY..:                                                      */
/*     Created 7.11.2002                                               */
/*                                                                     */
/***********************************************************************/

#include <isdc.h>		/* [DEF] ISDC subroutine parameters */
#include <spiback_observe.h>	/* [DEF] Observation pointing/count data structure */

 int spiback_parameter_values
(
 struct pointing_obs *obsv,	/* [OUT] Observation data parameters */
 char   ofil[],			/* [OUT] Filename of background component output dataset */
 char   phas[],			/* [OUT] Last parameter accessed before exiting */
 int    stat			/* [I-O] Subroutine exit status */
)

{				/* START OF SUBROUTINE CODE */

/*=======================================================*/
/*     W O R K I N G - S T O R A G E   D A T A           */
/*=======================================================*/
 char  *pntr;
 int    i,m,n;
 int    mmax = 5;
 int    nofc = 0;
 int    d,dmax,dnum;
 int    nofb = 0;
 int    orda = 0;
 int    nint = 0;
 double Dval;
 double Diff;
 double Parm[32];
 double Samp,Camp;
 double Sper,Spha;
 double Leng;
 char   dtyp[32] = " ";
 char   rply[4] = " ";
 char  *tpos,*trem;
 char   text[1024];
 char   name[64];

#define mmax 5
 char   modl[mmax][32] = {"POWL","SIN","COS","SIN+1","COS+1"};
 int    pnum[mmax] = {2,3,3,3,3};

/*-------------------------------------------------------*/
/*   Log start of parameter input.                       */
/*-------------------------------------------------------*/
 if (stat != ISDC_OK) return stat;

 RILlogMessage(NULL,Log_0," ");
 RILlogMessage(NULL,Log_0,"--------------------------------------------------------");
 RILlogMessage(NULL,Log_0,"<spiback_parameter_values> will read in input parameters");
 RILlogMessage(NULL,Log_0,"--------------------------------------------------------");

 obsv->nofb = 0;
 obsv->nofk = 0;
 obsv->Inti = 0.0;
 nofb = 0;

/*-------------------------------------------------------*/
/*   Get filename of output dataset.                     */
/*-------------------------------------------------------*/
 strcpy(phas,"back-mod-dol");		/* Get filename of output dataset */
 stat = PILGetString(phas,ofil);
 if (stat != ISDC_OK) return stat;

 if (!strlen(ofil))			/* Not present so create one  */
    {
    strcpy(phas,"back-mod-idx");	/* Look for name of index file */
    stat = PILGetString(phas,ofil);
    if (stat != ISDC_OK) return stat;	/* It must be present */

    pntr = strchr(ofil,'(');     if (pntr) strcpy(pntr,"\0");	/* Remove any (...) */
    pntr = strchr(ofil,'[');     if (pntr) strcpy(pntr,"\0");	/* Remove any [...] */
    pntr = strstr(ofil,".fits"); if (pntr) strcpy(pntr,"\0");	/* Remove any ".fits" */
    n = strlen(ofil);
    strcpy(ofil+n,"_data.fits");	/* Append to index file root name */
    RILlogMessage(NULL,Log_0,"Background output dataset not specified so it will be %s",ofil);
    }
 else
    {
    pntr = strchr(ofil,'('); if (pntr) strcpy(pntr,"\0");	/* Remove any (...) */
    pntr = strchr(ofil,'['); if (pntr) strcpy(pntr,"\0");	/* Remove any (...) */
    }

/*-------------------------------------------------------*/
/*   Set relative background levels in each detector     */
/*-------------------------------------------------------*/
 dnum = 0;
 m = 0;
 while((m < mmax) && (stat == ISDC_OK))	/* Read orbital model parameters for models listed in <modl> */
    {
    m++;

    if (m == 1)				/* Get relative background in single detectors */
       {
       strcpy(phas,"back-singles");
       dmax = 19;
       strcpy(dtyp,"single event");
       }
    else if (m == 2)			/* Get relative background in multiple detectors */
       {
       strcpy(phas,"back-multiples-1");
       dmax = 22;
       strcpy(dtyp,"multiple event");
       }
    else if (m == 3)			/* Get relative background in multiple detectors */
       {
       strcpy(phas,"back-multiples-2");
       dmax = 22;
       strcpy(dtyp,"multiple event");
       }
    else if (m == 4)			/* Get relative background in multiple detectors */
       {
       strcpy(phas,"back-multiples-3");
       dmax = 22;
       strcpy(dtyp,"multiple event");
       }
    else if (m == 5)			/* Get relative background in PSD event detectors */
       {
       strcpy(phas,"back-psdevents");
       dmax = 57;
       strcpy(dtyp,"PSD event");
       }

    RILlogMessage(NULL,Log_0," ");
    RILlogMessage(NULL,Log_0,"Setting relative background rates for %d %s detectors",dmax,dtyp);
    RILlogMessage(NULL,Log_0," ");

    if (m < mmax)			/* Get parameters for all but PSD event detectors */
       {
       stat = PILGetString(phas,text);	/* Get list of detector background multipliers */	
       if (stat != ISDC_OK) return stat;

       i = 0;
       tpos = text;
       trem = text;
       while(*tpos && (isdigit(*tpos) || !strncmp(tpos,".",1))) /* Extract <pnum[m-1]> orbital component parameters from string <text> */
          {
          i++;
          obsv->detb[dnum+i-1] = (float) strtod(tpos,&trem);
          tpos = trem;
          while(*tpos && !isdigit(*tpos) && strncmp(tpos,".",1)) tpos++;
          }

       if (i == 1)			/* Make all detector rates the same as the first */
          {
          i = dmax;
          for(d=1; d<=dmax; d++) obsv->detb[dnum+d-1] = obsv->detb[dnum];
          }
       }
    else				/* Append PSD relative rates */
       {
       i = dmax;
       for(d=0; d<19; d++) 
          {
          obsv->detb[dnum+d] = obsv->detb[d];
          obsv->detb[dnum+d+19] = obsv->detb[d];
          obsv->detb[dnum+d+38] = obsv->detb[d];
          }
       }

    for(d=0; d<dmax; d++)
       {
       if ((m == mmax) && (d == 19 || d == 38)) RILlogMessage(NULL,Log_0," ");
       RILlogMessage(NULL,Log_0,"Relative value for %s detector %d is %f",dtyp,d+dnum,obsv->detb[d+dnum]);
       }

    if (i < dmax)			/* Less than <dmax> values specified */
       {
       RILlogMessage(NULL,Log_0," ");
       RILlogMessage(NULL,Error_1,
       "No of relative background values specified for %s detectors are less than the %d required",dtyp,dmax);
       stat = PIL_VALUE_INVALID;
       return stat;
       }

    dnum = dnum + dmax;

    }

/*---------------------------------------------------------*/
/*   Is a general polynomial up to quartic required?       */
/*   NOTE: Polynomial coefficients are currently ignored.  */
/*---------------------------------------------------------*/
 strcpy(phas,"back-polynomial");		/* Get polynomial with form "TYPE,a,b,c,d,e" */
 stat = PILGetString(phas,text);		/* Examples are "CONSTANT,a", "LINEAR,a,b", "CUBIC,a,b,c,d" etc */
 if (stat != ISDC_OK) strcpy(text,"");
 stat = ISDC_OK;				/* Polynomial parameter is currently optional */

 RILlogMessage(NULL,Log_0," ");
 orda = 0;					/* Default polynomial order to zero */
   
 if (strlen(text) && !strncmp(text,"MEAN",4))	/* Mean count rate model has been selected */
    {
     nofb = 1;
     strcpy(obsv->bnam[nofb-1],text);
     obsv->nofb = nofb;
     obsv->orda[nofb-1] = 0;			/* Spline order 1=CONSTANT, 2=LINEAR etc */
     obsv->nofc[nofb-1] = 0;			/* There are no special parameters */
     obsv->Parm[nofb-1][0] = 1.0;		/* Multiply by 1.0 */
     RILlogMessage(NULL,Log_0,"Creating a mean count per pointing model",text);
     return stat;				/* Exit as thsi model exists by itself */
    }

 if (strlen(text) && strncmp(text,"N",1))	/* Polynomial component has been selected */
    {
    RILlogMessage(NULL,Log_0,"Background has a %s polynomial component.",text);

/*  --------------------------------------------------------------*/
/*  Set order and correct name of polynomial selected.            */
/*  --------------------------------------------------------------*/
    if (!strncmp(text,"CON",3))		/* CONSTANT */
       {
       orda = 1;
       strcpy(name,"CONSTANT");
       }
    else if (!strncmp(text,"LIN",3))	/* LINEAR polynomial */
       {
       orda = 2;
       strcpy(name,"LINEAR");
       }
    else if (!strncmp(text,"QUAD",4))	/* QUADRATIC polynomial */
       {
       orda = 3;
       strcpy(name,"QUADRATIC");
       }
    else if (!strncmp(text,"CUB",3))	/* CUBIC polynomial */
       {
       orda = 4;
       strcpy(name,"CUBIC");
       }
    else if (!strncmp(text,"QUAR",4))	/* QUARTIC polynomial */
       {
       orda = 5;
       strcpy(name,"QUARTIC");
       }
    else
       {
       RILlogMessage(NULL,Error_1,"Polynomial component %s not recognized",text);
       stat = PIL_VALUE_INVALID;
       return stat;
       }

/*  --------------------------------------------------------------*/
/*  Calculate no of spline sequences required or evaluate the     */
/*  string of polynomial coefficients plus MJD reference time.    */
/*  --------------------------------------------------------------*/
    if ( tpos = strstr(text,"-") )		/* Create a sequence of splines */
       {
       Leng = strtod(tpos+1,&trem);		/* Get spline node separation length in days */
       nint = 1;
       if (Leng > 0.0)				/* Calculate no of splining intervals */
          {
          nint = floor( (obsv->End - obsv->Start)/Leng );
          if (nint < 1) nint = 1;
          if (nint > 4000) nint = 4000;
          }

       nofb++;					/* Store splining component */
       strcpy(obsv->bnam[nofb-1],name);
       obsv->orda[nofb-1] = orda;		/* Spline order 1=CONSTANT, 2=LINEAR etc */
       obsv->nofc[nofb-1] = 1;			/* There are no special parameters */
       obsv->Parm[nofb-1][0] = 1.0;		/* Multiply splines by 1.0 */
       obsv->Inti = Leng;			/* Splining integration interval */
       obsv->nofk = nint;			/* No of spline intervals */
       RILlogMessage(NULL,Log_0,"Splining component %s will cover %d equal intervals.",name,nint);
       }
    else					/* Spline observation over one interval */
       {
       Parm[0] = 1.0;				/* Default polynomial to a constant value */
       for(i=1; i<orda; i++) Parm[i] = 0.0;	/* Init polynomial parameters to 1.0 */
       Parm[orda] = 0.0;			/* Default IJD reference time zero */
       Parm[orda+1] = obsv->Start;		/* IJD start time of observation period */
       Parm[orda+2] = obsv->End;		/* IJD end time of observation period */

       tpos = text;
       trem = text;
       while(*tpos && !isdigit(*tpos)) tpos++;	/* Find first digit */
       nofc = 0;
       i = 0;
       while(*tpos && isdigit(*tpos))		/* Extract polynomial coefficients in string <text> */
          {
          Parm[i] = strtod(tpos,&trem);		/* Polynomial coefficient */
          if (Parm[i] > 0.0) nofc++;		/* Count no of non zero coefficients */
          i++;
          tpos = trem;
          while(*tpos && !isdigit(*tpos)) tpos++;
          }

/*     -------------------------------------------------------------*/
/*     Set observation MJD reference, START, END times to IJDs.     */
/*     Transform from polynomial to B-spline coefficients.          */
/*     -------------------------------------------------------------*/
       if (Parm[orda] <= 0.0)			/* MJD reference time not given */
          {
          Parm[orda] = obsv->Start;		/* Reference time is IJD start time of observation period */
          RILlogMessage(NULL,Log_0,"IJD reference time defaulted to %f",Parm[orda]);
          }
       else if (Parm[orda] > obsv->Mjdref)	/* An MJD reference time is given by user */
          {
          Parm[orda] = Parm[orda] - obsv->Mjdref;	/* otherwise an IJD reference time is assumed */
          RILlogMessage(NULL,Log_0,"MJD reference changed to IJD time of %f",Parm[orda]);
          }
       else
          {
          RILlogMessage(NULL,Log_0,"IJD reference time given is %f",Parm[orda]);
          }

       nofb++;					/* Store polynomial component */
       strcpy(obsv->bnam[nofb-1],name);
       obsv->orda[nofb-1] = orda;		/* Spline order is zero */
       obsv->nofc[nofb-1] = orda;		/* No of spline multiplier coefficients */
       for(n=0; n<orda; n++)			/* Store coefficients for each B-spline */
          {
          obsv->Parm[nofb-1][n] = Parm[n];
          RILlogMessage(NULL,Log_0,"Coefficient of %s component %d has value %f",name,n+1,Parm[n]);
          }
       obsv->Inti = obsv->End - obsv->Start + 1;/* Splining integration interval */
       obsv->nofk = 1;				/* No of spline intervals is just one */

       }

    }

/*-------------------------------------------------------*/
/*  Choose CONSTANT and LINEAR ramp components only.     */
/*-------------------------------------------------------*/
 if (orda == 0)			/* No polynomial chosen above */
    {
/*  -----------------------------*/
/*  Choose CONSTANT component.   */
/*  -----------------------------*/
    strcpy(phas,"back-const");
    stat = PILGetString(phas,rply);
    if (stat != ISDC_OK) strcpy(rply,"N");
    stat = ISDC_OK;

    RILlogMessage(NULL,Log_0," ");
    Parm[0] = 0.0;
    Parm[1] = 0.0;

    if (!strncmp(rply,"y",1) || !strncmp(rply,"Y",1))
       {
       strcpy(phas,"back-const-value");
       stat = PILGetReal(phas,&Parm[0]);
       if (stat != ISDC_OK) return stat;

       if (Parm[0] <= 0.0)		/* Amplitude of CONSTANT component cannot be zero */
          {
          RILlogMessage(NULL,Error_1,"CONSTANT component must have a non zero amplitude.");
          stat = PIL_VALUE_INVALID;
          return stat;
          }

       orda = 1;
       nofb++;
       strcpy(obsv->bnam[nofb-1],"CONSTANT");
       obsv->orda[nofb-1] = 0;		/* Spline order is zero */
       obsv->nofc[nofb-1] = 1;
       obsv->Parm[nofb-1][0] = Parm[0];
       RILlogMessage(NULL,Log_0,"Background has a CONSTANT component with amplitude %f cnts/sec/kev/det.",Parm[0]);
       }

/*  -------------------------------*/
/*  Choose LINEAR ramp component.  */
/*  -------------------------------*/
    strcpy(phas,"back-linear");
    stat = PILGetString(phas,rply);
    if (stat != ISDC_OK) strcpy(rply,"N");
    stat = ISDC_OK;
   
    RILlogMessage(NULL,Log_0," ");

    if (!strncmp(rply,"y",1) || !strncmp(rply,"Y",1))
       {

       if (Parm[0] <= 0.0)			/* Amplitude of CONSTANT component is zero */
          {
          RILlogMessage(NULL,Error_1,"A LINEAR ramp component requires a CONSTANT component with non zero amplitude.");
          stat = PIL_VALUE_INVALID;
          return stat;
          }

       strcpy(phas,"back-linear-slope");	/* Get gradient of ramp */
       stat = PILGetReal(phas,&Parm[1]);
       if (stat != ISDC_OK) return stat;

       strcpy(phas,"back-linear-mjd0");		/* MJD reference time */
       stat = PILGetReal(phas,&Parm[2]);
       stat = ISDC_OK;

       RILlogMessage(NULL,Log_0,
       "Background has a LINEAR component with height %f and slope of %f cnts/sec/det/keV/day.",
       Parm[0],Parm[1]);

       Diff = obsv->End - obsv->Start;
       strcpy(obsv->bnam[nofb-1],"LINEAR");	/* Reset CONSTANT component to LINEAR */
       obsv->nofc[nofb-1] = 2;
       obsv->Parm[nofb-1][0] = Parm[0];
       obsv->Parm[nofb-1][1] = Parm[0] + Diff*Parm[1];

       }

    }

/*-------------------------------------------------------*/
/*   Select a SINUSOIDAL component.                      */
/*-------------------------------------------------------*/
 strcpy(phas,"back-sincos");
 stat = PILGetString(phas,rply);	/* Select SINE/COSINE components */
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"back-sin-amp");
 stat = PILGetReal(phas,&Samp);		/* Get SINE amplitude */
 if (stat != ISDC_OK) return stat;

 strcpy(phas,"back-cos-amp");		/* Get COSINE amplitude */
 stat = PILGetReal(phas,&Camp);
 if (stat != ISDC_OK) return stat;
    
 if ((Samp == 0.0 )&&(Camp == 0.0)) strcpy(rply,"NO");	/* Must have an amplitude */

 RILlogMessage(NULL,Log_0," ");

 if (!strncmp(rply,"y",1) || !strncmp(rply,"Y",1))	/* SINE/COSINE component selected */
    {

    strcpy(phas,"back-sincos-period");	/* Get SINE/COSINE period */
    stat = PILGetReal(phas,&Sper);
    if (stat != ISDC_OK) return stat;

    if (Sper == 0.0)			/* Period must be non zero */
       {
       RILlogMessage(NULL,Log_0,"Period of SINUSOIDAL background component is zero.");
       stat = PIL_VALUE_INVALID;
       return stat;
       }

    strcpy(phas,"back-sincos-mjd0");	/* Get SINE/COSINE MJD phase */
    stat = PILGetReal(phas,&Spha);
    if (stat != ISDC_OK) return stat;

    if (Samp != 0.0)			/* SINE component selected */
       {
       RILlogMessage(NULL,Log_0,
       "Background has a SINE component with amplitude %f cnts/sec/kev/det, period %f days and MJD start-time %f",Samp,Sper,Spha);
       nofb++;
       strcpy(obsv->bnam[nofb-1],"SINE");
       obsv->orda[nofb-1] = 0;		/* Spline order is zero */
       obsv->nofc[nofb-1] = 3;		/* No of SINE parameters */
       obsv->Parm[nofb-1][0] = Spha;	/* SINE phase or MJD start-time */
       obsv->Parm[nofb-1][1] = Samp;	/* SINE amplitude */
       obsv->Parm[nofb-1][2] = Sper;	/* SINE period in days */
       }

    if (Camp != 0.0)			/* COSINE component selected */
       {
       RILlogMessage(NULL,Log_0,
       "Background has a COSINE component with amplitude %f cnts/sec/kev/det, period %f days and MJD start-time %f",Camp,Sper,Spha);
       nofb++;
       strcpy(obsv->bnam[nofb-1],"COSINE");
       obsv->orda[nofb-1] = 0;		/* Spline order is zero */
       obsv->nofc[nofb-1] = 3;		/* No of COSINE parameters */
       obsv->Parm[nofb-1][0] = Spha;	/* COSINE phase or MJD start-time */
       obsv->Parm[nofb-1][1] = Camp;	/* COSINE amplitude */
       obsv->Parm[nofb-1][2] = Sper;	/* COSINE period in days */
       }

    }

 else

    {
    RILlogMessage(NULL,Log_0,"There is no SINUSOIDAL background model component");
    }

/*-------------------------------------------------------*/
/*   Any components proportional to SCHK count rates?    */
/*-------------------------------------------------------*/
 strcpy(phas,"back-SCHK");
 stat = PILGetString(phas,text);
 if (stat != ISDC_OK) return stat;

 RILlogMessage(NULL,Log_0," ");

 if (!strlen(text) || !strncmp(text,"N",1))
    {
    RILlogMessage(NULL,Log_0,"There is no background model based on Science House Keeping data");
    strcpy(text," ");
    strcpy(&obsv->onegtiseq,"N");
    }
/*  ------------------------------------------------------------------*/
/*  Is it assumed the GTI sequence is the same for all detectorsi.    */
/*  ------------------------------------------------------------------*/
 else
    {
    strcpy(phas,"back-onegtiseq");
    stat = PILGetString(phas,rply);
    if (stat != ISDC_OK) return stat;

    if (!strlen(rply) || !strncmp(rply,"Y",1))
       {
       RILlogMessage(NULL,Log_0,"Any pointing GTI sequence is assumed to be the same for all detectors.");
       strcpy(&obsv->onegtiseq,"Y");
       }
    else
       {
       RILlogMessage(NULL,Log_0,"Any pointing GTI sequence is assumed to be different for each detector.");
       strcpy(&obsv->onegtiseq,"N");
       }
    }

 pntr = strstr(text,"ACS"); 

 if (pntr)
    {
    nofb++;				/* Update background model counter */
    strncpy(obsv->bnam[nofb-1],pntr,3);	/* Component name */
    obsv->orda[nofb-1] = 0;		/* Spline order is zero */
    obsv->nofc[nofb-1] = 0;		/* Default no of extra parameters to zero */
    RILlogMessage(NULL,Log_0," ");
    RILlogMessage(NULL,Log_0,"Background has a SCHK archived time variable ACS component");
    }

 pntr = strstr(text,"IREM"); 
 if (pntr)
    {
    nofb++;				/* Update background model counter */
    strncpy(obsv->bnam[nofb-1],pntr,4);	/* Component name */
    obsv->orda[nofb-1] = 0;		/* Spline order is zero */
    obsv->nofc[nofb-1] = 0;		/* Default no of extra parameters to zero */
    RILlogMessage(NULL,Log_0," ");
    RILlogMessage(NULL,Log_0,"Background has a SCHK archived time variable IREM component");
    }

 pntr = strstr(text,"DFEE"); 
 if (pntr)
    {
    nofb++;				/* Update background model counter */
    strncpy(obsv->bnam[nofb-1],pntr,4);	/* Component name */
    obsv->orda[nofb-1] = 0;		/* Spline order is zero */
    obsv->nofc[nofb-1] = 0;		/* Default no of extra parameters to zero */
    RILlogMessage(NULL,Log_0," ");
    RILlogMessage(NULL,Log_0,"Background has a SCHK archived time variable DFEE component");
    }

/*-------------------------------------------------------*/
/*   Is an orbital model required.                       */
/*-------------------------------------------------------*/
 m = 0;
 while((m < mmax) && (stat == ISDC_OK))		/* Read orbital model parameters for models listed in <modl> */
 {
    m = m + 1;
    strcpy(phas,"back-orbital-");		/* Append orbital component name to orbital parameter root name */
    n = strlen(phas);
    strcpy(phas+n,modl[m-1]);			/* Append orbital component name */

    stat = PILGetString(phas,text);
    if (stat != ISDC_OK) return stat;

    RILlogMessage(NULL,Log_0," ");
   
    if (strlen(text) && strncmp(text,"NO",2))	/* Orbital component <modl[m-1]> has been selected */
       {
        RILlogMessage(NULL,Log_0,"Background ORBITAL component %s has been selected",modl[m-1]);

        nofb++;					/* Update background model counter */
        obsv->orda[nofb-1] = 0;			/* Spline order is zero */
        obsv->nofc[nofb-1] = 0;			/* Default no of extra parameters to zero */
        strcpy(obsv->bnam[nofb-1],modl[m-1]);	/* Orbital model component name */

        i = 0;
        tpos = text;
        trem = text;
        while(*tpos && isdigit(*tpos))		/* Extract <pnum[m-1]> orbital component parameters from string <text> */
           {
           Dval = strtod(tpos,&trem);		/* Orbital model parameter */
           i++;
           obsv->nofc[nofb-1] = i;		/* Default no of extra parameters to zero */
           obsv->Parm[nofb-1][i-1] = Dval;	/* Orbital model parameter */
           RILlogMessage(NULL,Log_0,"Background ORBITAL component %s has parameter %d with value %f",modl[m-1],i,Dval);
           tpos = trem;
           while(*tpos && !isdigit(*tpos)) tpos++;
           }

        if (i < pnum[m-1])			/* Less than <pnum[m-1]> parameters specified */
           {
           RILlogMessage(NULL,Error_1,"Background ORBITAL component %s has less than %d parameters",modl[m-1],pnum[m-1]);
           stat = PIL_VALUE_INVALID;
           return stat;
           }

        if (obsv->nofc[nofb-1] < 1)
           {
           RILlogMessage(NULL,Log_0,"Background ORBITAL component %s has no parameters",modl[m-1]);
           }

       }

    else

       {
       RILlogMessage(NULL,Log_0,"There is no background ORBITAL component of type %s",modl[m-1]);
       }
 }

 obsv->nofb = nofb;
 RILlogMessage(NULL,Log_0," ");
   
 return stat;
}

