/***********************************************************************/
/*                                                                     */
/*      SUBROUTINE: spiback_polynomial_model                           */
/*                                                                     */
/*      DESCRIPTION:                                                   */
/*      Returns observation counts binned by detector,pointing,energy. */
/*                                                                     */
/***********************************************************************/
/*                                                                     */
/*      AUTHOR  : P.H.Connell - phc@star.sr.bham.ac.uk                 */
/*      ADDRESS : University of Birmingham - Space Research Group      */
/*                                                                     */
/*      HISTORY : Created 10.11.2002                                   */
/*                                                                     */
/***********************************************************************/

#include <isdc.h>		/* [DEF] ISDC int parameters */
#include <spiback_observe.h>	  /* [DEF] Observation pointing/count data structure */

 int spiback_polynomial_model   
(
 struct pointing_obs *obsv,	/* [IN]  Observation pointing/count header */
 float  pcnt[],			/* [IN]  Observation [D,P,E] count spectra */
 float  pvar[],			/* [IN]  Observation [D,P,E] count variance spectra */
 double MJDs[],			/* [IN]  Observation [I,D,P] exposure GTI startimes */
 double MJDe[],			/* [IN]  Observation [I,D,P] exposure GTI endtimes */
 double LIVETIME[],		/* [IN]  Observation [D,P] exposure livetimes */
 double C[]			/* [OUT] Polynomial coefficients */
)

{				/* START OF SUBROUTINE */

/*=====================================================================*/
/*      W O R K I N G - S T O R A G E    D A T A                       */
/*=====================================================================*/
#define nmax 10000		/* [DEF] Maximum no of sampling points expected */
#define cmax 3			/* [DEF] Order of polynomial */
 LogFile LFIL = NULL;
 static char *subr = "spiback_polynomial_model";
 int    d,p,e,q;
 int    i,j,k,n;
 int    stat = 0;
 double *Rate;
 double *Time;
 double Expo;
 double Cnts;
 double Chi2;

 int    poly;			/* No of Ratetion sampling points */
 double X[nmax][cmax];		/* Matrix relating values in <Rate> to sampling point */
 double XtrX[cmax][cmax];	/* [X] multiplied by its transpose */
 double XtrF[cmax];		/* <Rate> multiplied by the transpose of [X] */
 double Xsum;
 double A,B;
 
/*=====================================================================*/
/*   Int count rate accumulation buffer.                               */
/*=====================================================================*/
 stat = ISDC_OK;
 poly = cmax;
 if (obsv->nofp == 1) poly = 1;
 if (obsv->nofp == 2) poly = 2;
 if (obsv->nofp == 3) poly = 3;
 Rate = malloc(obsv->nofp*sizeof(double));
 Time = malloc(obsv->nofp*sizeof(double));

 for(p=0; p<obsv->nofp; p++)		/* Scan count pointing bins */
    {
    Rate[p] = 0.0;
    }

/*=====================================================================*/
/*  Get count spectra integer values and output as a real-no vector.   */
/*=====================================================================*/
 q = 0;
 for(e=0; e<obsv->nofe; e++)		/* Scan count energy bins */
    {
    n = 0;
    for(p=0; p<obsv->nofp; p++)		/* Scan count pointing bins */
       {
       for(d=0; d<obsv->nofd; d++)	/* Scan count detector bins */
          {
          Expo = LIVETIME[n];
          Cnts = (double)pcnt[q];
          if (Expo > 0.0) Rate[p] = Rate[p] + Cnts/Expo;	/* Accumulate count per pointing */
          q++;
          n++;
          }
       }
    }

/*====================================================================*/
/*   Set up the polynomial interpolation matrix [X] such that         */
/*   Rate(k) = X(k,i)*C(i) and C(i) is polynomial coefficient vector. */
/*====================================================================*/
 for(p=0; p<obsv->nofp; p++)		/* Scan count pointing bins */
    {
    Time[p] = 0.5*(MJDe[p] + MJDs[p]);

    X[p][0] = (double)1.0;		/* First column is always 1.0 */
    A = Time[p] - obsv->Start;		/* Sampling ordinate */
    B = A;
    for(i=1; i<poly; i++)		/* Count columns across [X] */
       {
       X[p][i] = A;			/* Element value is <A**(i-1)> */
       A = A*B;				/* Multiply A to its next power */
       }
    }

/*====================================================================*/
/*   Set up square LSQ matrix equation to determine coefficients.     */
/*          t       t                                                 */
/*   i.e. [X]*F = [X][X]*C.                                           */
/*====================================================================*/
 for(i=0; i<poly; i++)			/* Count rows down matrix [X] */
    {

    Xsum = 0.0;				/* First get vector XtrF */
    for(p=0; p<obsv->nofp; p++)		/* Count columns across [X] */
       {
       Xsum = Xsum + X[p][i]*Rate[p];
       }
    XtrF[i] = Xsum;

    for(j=0; j<poly; j++)		/* Count columns across [X] */
       {
       Xsum = 0.0;
       for(p=0; p<obsv->nofp; p++)	/* Count columns across [X] */
          {
          Xsum = Xsum + X[p][i]*X[p][j];
          }
       XtrX[i][j] = Xsum;		/* Elements of [XtrX] Hessian matrix */
       }

    }

/*====================================================================*/
/*   Solve the above matrix equation simultaneously.                  */
/*====================================================================*/
 for(i=0; i<poly; i++)
    {

    A = XtrX[i][i];
    XtrF[i] = XtrF[i]/A;
    for(j=i; j<poly; j++)
       {
       XtrX[i][j] = XtrX[i][j]/A;
       }

    for(k=0; k<poly; k++)
       {
       B = XtrX[k][i];
       if (k != i)
          {
          XtrF[k] = XtrF[k] - XtrF[i]*B;
          for(j=0; j<poly; j++)
             {
             XtrX[k][j] = XtrX[k][j] - XtrX[i][j]*B;
             }
          }
       }

    }
 
 for(i=0; i<poly; i++) {C[i] = XtrF[i];}

 Chi2 = 0.0;
 for(p=0; p<obsv->nofp; p++)		/* Scan count pointing bins */
    {
    Xsum = C[0];
    A = Time[p] - obsv->Start;		/* Sampling ordinate */
    for(i=1; i<poly; i++)		/* Count columns across [X] */
       {
       Xsum = Xsum + C[i]*A;		/* Element value is <A**(i-1)> */
       A = A*A;
       }
    B = Rate[p] - Xsum;
    Chi2 = Chi2 + B*B;
    }

 RILlogMessage(LFIL,Log_0,"");
 RILlogMessage(LFIL,Log_0,
 "Observation counts vary as %f + %f*T + %f*T*T where T is time from IJD %f",
 C[0],C[1],C[2],obsv->Start);

 return stat; 
}
