/********************************************************************/
/*                                                                  */
/*    FUNCTION: spiback_polynomial_to_bspline                       */
/*                                                                  */
/*    DESCRIPTION:                                                  */
/*    Returns the coefficients of polynomial B-spline components    */
/*    that produce a fit to polynomial f(t) = c1 + c2*t + c3*t*t..  */
/*                                                                  */
/*    For a polynomial of degree <n> coefficients an additional     */
/*    n+1, n+2 and n+3 values  will contain an MJD reference, start */
/*    and end time of the period of observation.                    */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/*    AUTHOR.: P.H.Connell - phc@star.sr.back.ac.uk                 */
/*    ADDRESS: University of Birmingham - Space Research Group      */
/*                                                                  */
/*    HISTORY: Created 31.1.2003                                    */
/*                                                                  */
/********************************************************************/

#include <math.h>	/* [DEF] Mathematical functions */
#include <isdc.h>	/* [DEF] ISDC subroutine parameters */

 int spiback_polynomial_to_bspline
 (
 double Poly_coef[],	/* [IN] Vector of polynomial coefficients */
 int    degr,		/* [IN] Polynomial degree [1,2,3,4,5] */
 double Bspl_coef[] 	/* [OUT] B-spline coefficients */
 )

{			/* Start of subroutine algorithm */

/*==================================================================*/
/*    W O R K I N G - S T O R A G E   D A T A                       */
/*==================================================================*/
 const  int NO_POLYNOMIAL_TO_BSPLINE = -210034;
 const  double Dzero = 0.0;
 int    stat = 0;
 int    i,m,n;
 double F[32];
 double X[32][32];
 double Dval;
 /* double Indx; */
 double MJDt;
 double MJDs,MJDe,MJDw;
 double MJDr;
 /* double Refr; */
 const  float wdth = 1.0;
 float  frac;
 float  hght;
 double  Hght;

/*==================================================================*/
/*    Create B-spline/Polynomial sampling matrix equation.          */
/*    F(i) = X(i,j)*B(j) j=1,degr is ith sampling value.            */
/*==================================================================*/
 MJDs = Poly_coef[degr+1];
 MJDe = Poly_coef[degr+2];
 MJDw = MJDe - MJDs;
 MJDr = MJDs - Poly_coef[degr];

 for (n=0; n<degr; n++)
     {

     frac = (float)n/(degr-1);	/* (n+1)th sampling point over Observation period */

     MJDt = MJDr + MJDw*(double)frac;
     Dval = 1.0;
     F[n] = Poly_coef[0];

     for (m=1; m<degr; m++)	/* Get polynomial value at (n+1)th sampling point */
         {
         Dval = Dval*MJDt;
         F[n] = F[n] + Poly_coef[m]*Dval;
         }

     for (m=0; m<degr; m++)	/* Get contributions from B-spline components */
         {
         stat = spiback_bspline_function(degr,1,m+1,&wdth,&frac,&hght);
/*       stat = spiback_bspline_function(degr,obsv->splo[s-1],obsv->Knot,&Dval,&Hght); */
         X[n][m] = Hght;
         }

     }

/*==================================================================*/
/*    Solve B-spline/Polynomial sampling matrix equation for        */
/*    B-spline coefficients using Gaussian elimination to reduce    */
/*    matrix [X] to an identity matrix.                             */
/*==================================================================*/
 for (n=0; n<degr; n++)			/* Scan diagonal of matrix [X] */
     {
     Dval = X[n][n];			/* Diagonal of [X] */
     if (Dval > Dzero)			/* Divide equation <n> by diagonal X[n][n] */
        {
        F[n] = F[n]/Dval;		/* Divide sampled values by X[n][n] */
        for (m=0; m<degr; m++)		/* Divide row <n> of [X] by diagonal X[n][n] */
            { X[n][m] = X[n][m]/Dval; }

        for (i=0; i<degr; i++)		/* Scan down rows of [X] */
            {
            Dval = X[i][n];		/* Store value of elements in column [n] of [X] */
            if (i != n)			/* Only rows above and below row [n] */
               {
               F[i] = F[i] - F[n]*Dval;	/* Subtract <Dval> times sample [n] from sample [i] */
               for (m=n; m<degr; m++)	/* Subtract <Dval> times row [n] from row [i] */
                   { X[i][m] = X[i][m] - X[n][m]*Dval; }
               }
            }
        }
     else				/* Diagonal element cannot be zero - abort */
        {
        stat = NO_POLYNOMIAL_TO_BSPLINE;
        RILlogMessage(NULL,Error_1,
        "<spiback_polynomial_to_bspline> cannot convert a polynomial to bsplines.");
        return stat;
        }
     }

 for (n=0; n<degr; n++) Bspl_coef[n] = F[n];	/* Output vector F[n] as B-spline coefficients */

 return 0;
}
